/*
	copie une portion d'une chaine dans une autre
*/
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "etc.h"

/*
	copie un mot d'une chaine str dans une chaine dest
	Un mot est une suite de caractre contigue rpondant au critre isgraph()
	Saute les blancs au dbut de str
	Retourne l'adresse du premier caractre aprs le mot dans la chaine
	Retourne NULL si la chaine ne contenait pas de mot.
	La chaine dest est toujours termine mme s'il n'y a pas de mot.
*/
export char *str_copymot (
	char *dest,		/* recoit le mot */
	const char *str)
{
	char *ret = NULL;
	while (isspace(*str)) str++;
	if (*str > ' '){
		while (*str>' ') *dest++ = *str++;
		ret = (char*) str;
	}
	*dest = '\0';
	return (ret);
}

/*
	copie le n'ime mot d'une chaine str dans une chaine dest
	Un mot est une suite de caractre contigue rpondant au critre isgraph()
	Saute les blancs au dbut de str
	Retourne l'adresse du premier caractre aprs le mot dans la chaine
	Retourne NULL si la chaine ne contenait pas de nomot mot.
	La chaine dest est toujours termine mme s'il n'y a pas de mot.
*/
export char *str_copymotn (
	char *dest,		/* recoit le mot */
	const char *str,
	int nomot)		/* Numro du mot, 0 ... */
{
	while ((str=str_copymot(dest,str))!=NULL && nomot > 0) nomot--;
	return (char*)str;
}
#if 0
/*
	copie une squence englob entre deux caractre quote.
	Le premier caractre de str est le caractre quote.

	Deux caractres quote qui se suivent seront interprter comme un
	seul et incorpor dans la squence.

	Le caractre escape est gnralement '\\'.

	Retourne l'adresse du premier caractre aprs le caractre quote de
	fermeture dans str.

	UN PEU REDONDANT par rapport  lexc\token_copystring (sauf compilation)

	Les caractres quotes ne sont pas copis.
*/

export char *str_copyquotec (
	char *dest,		/* contiendra la squence entre les deux caractres */
					/* quote. Peut tre NULL, ca permet de trouver */
					/* la fin */
	const char *str,/* chaine commenant par un caractre quote */
	char escape,	/* Gnralement '\\' ou '\0' si absent */
	int compile)	/* Interprte les escapes ou laisse comme a */
{
	char quote = *str++;
	char dummy[10];
	int copie = 1;
	if (dest == NULL){
		dest = dummy;
		copie = 0;
	}
	while (*str != '\0'){
		if (!copie) dest = dummy;
		if (*str == quote){
			str++;
			if (*str == quote){
				if (!compile) *dest++ = quote;
				*dest++ = *str++;
			}else{
				break;
			}
		}else if (*str == escape){
			str++;
			if (compile){
				char res=*str;
				switch (*str){
				case 'b':
					res = '\b';
					break;
				case 'f':
					res = '\f';
					break;
				case 'n':
					res = '\n';
					break;
				case 'r':
					res = '\r';
					break;
				case 'a':
					res = '\a';
					break;
				default:
					if (isdigit(*str)){
						int base = 8;
						while (1){
							char carac = *str;
							if (isdigit(carac)){
								res = (char)(res * base + carac - '0');
							}else if (carac == 'x' || carac == 'X'){
								base = 16;
							}else if (isxdigit(carac) || base == 16){
								res = (char)(res * base + (carac & 0xf) + 9);
							}else{
								break;
							}
						}
					}
				}
				str++;
				*dest++ = res;
			}else{
				*dest++ = escape;
				if (*str != '\0') *dest++ = *str++;
			}
		}else{
			*dest++ = *str++;
		}
	}
	*dest = '\0';
	return (char*)str;
}

/*
	copie une squence englob entre deux caractre quote.
	Le premier caractre de str est le caractre quote.

	Deux caractres quote qui se suivent seront interprter comme un
	seul et incorpor dans la squence.

	Retourne l'adresse du premier caractre aprs le caractre quote de
	fermeture dans str.

	Les caractres quotes ne sont pas copis.
*/

export char *str_copyquote (
	char *dest,		/* contiendra la squence entre les deux caractres */
					/* quote. Peut tre NULL, ca permet de trouver */
					/* la fin */
	const char *str)/* chaine commenant par un caractre quote */
{
	return str_copyquotec(dest,str,'\0',1);
}
#endif
/*
	Copie une expression entre parenthse avec sous parenthse.
	Retourne adresse aprs la parenthse de fermeture.

	*ptaccum recevra l'expression incluant les parenthses.
*/
export char *str_copypar0 (
	char **ptaccum,		/* Pointera aprs parenthse de fermeture */
	const char *str)
{
	char *accum = *ptaccum;
	*accum++ = *str++;
	while (*str != '\0' && *str != ')'){
		if (*str == '('){
			str = str_copypar0 (&accum,str);
		}else{
			*accum++ = *str++;
		}
	}
	if (*str != '\0') *accum++ = *str++;
	*accum = '\0';
	*ptaccum = accum;
	return (char*) str;
}
/*
	Copie une expression entre parenthse avec sous parenthse.
	Retourne adresse aprs la parenthse de fermeture.

	accum recevra l'expression incluant les parenthses.
*/
export char *str_copypar (char *accum, const char *str)
{
	return str_copypar0 (&accum,str);
}

