#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "etc.h"

/*
	elimine les elements semblables d'une liste de chaine trie
	Retourne nombre d'element qui reste dans tb[].
	Les elements restant dans tb[] sont contigus
	Les elements redondant sont librs par free().
*/

export int tbstr_fusion (
	char *tb[],
	int nbstr)
{
	{
		/*
			elimine les elements semblables
		*/
		int i;
		char *premier = tb[0];
		for (i=1; i<nbstr; i++){
			char *pt = tb[i];
			if (strcmp (premier,pt)!=0){
				premier = pt;
			}else{
				free (pt);
				tb[i] = NULL;
			}
		}
	}
	{	
		/*
			elimine les trous
		*/
		int i;
		int to=0;
		for (i=0; i<nbstr; i++){
		 	if (tb[i] != NULL){
				tb[to++] = tb[i];
			}
		}
		nbstr = to;
	}
	return (nbstr);
}

/*
	Elimine un chaine de caractre d'une liste
	La chaine est dsinsre, le reste de la liste est copi par dessus
	Retourne un pointeur vers la chaine localise ou NULL
*/
export char *tbstr_remove (
	char *tbstr[],			/* table de chaine */
	int   nbstr,			/* nombre de chaine */
	const char *str)		/* chaine a localiser et eliminer */
{
	char *ret = NULL;
	int i;
	for (i=0; i<nbstr; i++){
		if (strcmp(tbstr[i],str)==0){
			ret = tbstr[i];
			memmove (tbstr+i,tbstr+i+1,(nbstr-i-1)*sizeof(char*));
			break;
		}
	}
	return (ret);
}

/*
	Elimine un chaine de caractre d'une liste et libere espace mmoire
	La chaine est dsinsre, le reste de la liste est copi par dessus
	Retourne le nouveau nombre de chaine dans la liste
*/
export int tbstr_delfree (
	char *tbstr[],			/* table de chaine */
	int   nbstr,			/* nombre de chaine */
	const char *str)		/* chaine a localiser et eliminer */
{
	char *remstr = tbstr_remove (tbstr,nbstr,str);
	if (remstr != NULL){
		nbstr--;
		free (remstr);
	}
	return (nbstr);
}

#ifdef TEST

static void free (void *pt)
{
	printf ("## libere :%s:\n",pt);
}

static void test (char *msg, char *tbstr[], int nbstr)
{
	char *tb[400];
	int i;
	int nb;
	printf ("-------- %s ----------\n",msg);
	for (i=0; i<nbstr; i++) tb[i] = strdup(tbstr[i]);
	printf ("------- avant -------\n");
	for (i=0; i<nbstr; i++){
		printf ("%3d :%s:\n",i,tb[i]);
	}
	nb = tbstr_fusion (tb,nbstr);
	printf ("------- apres -------\n");
	for (i=0; i<nb; i++){
		printf ("%3d :%s:\n",i,tb[i]);
	}
}

static char *tb1[]={
	"0","0","1","2","2","3","3","3"
};
#define NB1 (sizeof(tb1)/sizeof(char*))

static char *tb2[]={
	"0","1","2","2","3","3","3"
};
#define NB2 (sizeof(tb2)/sizeof(char*))
static char *tb3[]={
	"0","0","0","0","0","0","0"
};
#define NB3 (sizeof(tb3)/sizeof(char*))
static char *tb4[]={
	"0"
};
#define NB4 0


static void testr (
	char *tbstr[],
	int  nbstr,
	char *mot)
{
	int i;
	printf ("enleve :%s: dans\n",mot);
	for (i=0; i<5; i++){
		printf ("%s :%s:\n",i<nbstr ? "   " : "fin" , tbstr[i]);
	}
	printf ("tbstr_remove -> %s\n",tbstr_remove(tbstr,nbstr,mot));
}

void main (void)
{
	static char *tbr[]={
		"0","allo","bonjour","xyz","bouge pas"
	};
	testr (tbr,4,"allo");
	testr (tbr,3,"bonjour");
	testr (tbr,2,"xxx");

	test ("test 1",tb1,NB1);
	test ("test 2",tb2,NB2);
	test ("test 3",tb3,NB3);
	test ("test 4",tb4,NB4);
}
#endif

