/*
	Interprtation des commandes de prprocesseur
*/
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <tool.h>
#include <etc.h>
#include <misc.h>
#include "lexc.h"

static char precond_getpragma=0;

/*
	Enregistre si les pragma sont gnrs comme des tokens.
	Par dfaut, les #pragma sont simplement sauts.
*/
export void precond_pragmamode (int mode)
{
	precond_getpragma=(char)mode;
}

/*
	Extrait la commande de prprocesseur du dbut d'une ligne

	La commande n'est pas ncessairement coll sur le #
*/
static void near preproc_getcmd (
	char **ptl,			/* Pointe apres #, pointera sur argument */
	char *cmd)			/* contiendra la commande */
{
	char *line = str_skip(*ptl);
	while (isalpha(*line) || *line == '_') *cmd++ = *line++;
	*cmd = '\0';
	line = str_skip (line);
	*ptl = line;
}
/*
	Traite une ligne qui dbute par #
	Si le prprocesseur n'est pas actif, seul certaine commandes sont
	traites.
	Cette ligne est:
			un #define
			un #undef
			un conditionnel
			un #include
			un #include_next
			ou quelques chose d'inconnu
	Retourne la position de lecture dans la ligne
*/

char *preproc_anal (
	char *line,		/* buffer de ligne */
	char *ptl,		/* Pointe apres le # */
	int preproc,	/* Indique si preprocesseur est actif */
	TOKEN *tok)		/* Permet d'enregistrer les pragma */
{
	extern int precond_actif;
	int err = 0;
	char cmd[100];
	int is_include_next=0;
	lexcerr_lockinfo ();
	preproc_getcmd (&ptl,cmd);
	if (strcmp(cmd,"else")==0){
		ptl = precond_else (line,ptl);
	}else if (strcmp(cmd,"endif")==0){
		ptl = precond_endif (line,ptl);
	}else if (strcmp(cmd,"elif")==0){
		ptl = precond_elif (line,ptl);
	}else if (strcmp(cmd,"ifdef")==0){
		ptl = precond_ifdef (line,ptl,0);
	}else if (strcmp(cmd,"ifndef")==0){
		ptl = precond_ifdef (line,ptl,1);
	}else if (strcmp(cmd,"if")==0){
		ptl = precond_if (line,ptl);
	}else if (precond_actif){
		if(strcmp(cmd,"define")==0){
			ptl = preproc_recdefine (line,ptl);
		}else if (strcmp(cmd,"pragma")==0){
			if (precond_actif && precond_getpragma){
				/* #Spcification: prprocesseur / #pragma
					Optionnellement, les noncs #pragma sont
					gnrs comme des tokens. Seul le dbut de l'nonc
					jusqu'au premier commentaire, est copis dans le token.
				*/
				char *pttxt = token_alloctxt();
				char *ptcom = ptl;
				tok->longtxt = pttxt;
				while (*ptcom != '\0'){
					if (ptcom[0] == '/'
						&& (ptcom[1] == '*' || ptcom[1] == '/')){
						/* Commentaire trouv */
						break;
					}
					*pttxt++ = *ptcom++;
				}
				*pttxt++ = '\0';
				token_settxt (pttxt);
				tok->type = TOK_PRAGMA;
			}
			precond_skipline(&ptl,line);
		}else if (cmd[0] == '\0'
			|| strcmp(cmd,"ident")==0){
			precond_skipline(&ptl,line);
		}else if (strcmp(cmd,"include")==0
			|| (is_include_next=(strcmp(cmd,"include_next")==0)) != 0){
			/*
				Il faut sauter les commentaires  la fin
				de la ligne avant de changer de fichier courant.
				Le commentaire peut continuer sur la ligne suivante.
			*/
			if (preproc){
				char fname[MAXSIZ_PATH];
				char *pt = fname;
				while (isgraph(*ptl)) *pt++ = *ptl++;
				pt--;
				//precond_skipline(&ptl,line);
				//comment_getlast();		/* Oublie le commentaire */
				if (is_include_next) is_include_next = 2;
				if (fname[0] == '<' && *pt == '>'){
					*pt = '\0';
					filesrc_pushopen (fname+1,is_include_next+1,ptl,line);
				}else if (fname[0] == '"' && *pt == '"'){
					*pt = '\0';
					filesrc_pushopen (fname+1,is_include_next,ptl,line);
				}else{
					err = 1;
				}
			}else{
				precond_skipline(&ptl,line);
			}
		}else if (strcmp(cmd,"undef")==0){
			ptl = preproc_undef (line,ptl);
		}else if (strcmp(cmd,"error")==0){
			char *tmpl = strdup(ptl);
			char *debpt = ptl;
			char *endpt = precond_skipline (&ptl,line);
			tmpl[(int)(endpt-debpt)] = '\0';
			lexcerr_bprintf ("Directive d'erreur : %s\n"
				,"User error : %s\n",tmpl);
			free (tmpl);
		}else{
			err = 1;
			precond_skipline(&ptl,line);
		}
	}else{
		precond_skipline(&ptl,line);
	}
	if (err){
		lexcerr_bprintf ("Commande invalide : %s\n"
			,"Invalid preprocessor command : %s\n",cmd);
	}
	lexcerr_unlock ();
	return (ptl);
}

