/*
	Lecture des token dans fichier
*/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>
#include <etc.h>
#include <misc.h>
#include "lexc.h"

extern int filesrc_noline;
extern int precond_actif;

/*
	initialise package token
*/
static char *ptl;
static char *fullbuf;		/* tampon de lecture et expansion des macros */
static char *line;			/* position de lecture des nouvelles lignes */
							/* dans fullbuf */
static TOKEN utok;
static int   flg_unget=0;
/*
	Initialise la collecte de token
	Par defaut, la lecture des tokens se fait sur stdin
*/
export void token_init ()
{
	if (line == NULL){
		fullbuf = (char*)malloc_err (10000,1);
		line = fullbuf+5000;
	}
	token_inittxt();
	comment_init();
    filesrc_setstdin ();
    line[0] = '\0';
    ptl = line;
	flg_unget = 0;
	preproc_init ();
	token_inithash();
}

static int preproc;
/*
	Enregistre le mode d'activit du prprocesseur
	0 : pas actif
	1 : actif 	->	traitement des includes
					traitement des defines
					traitement des #if
	Par dfaut pas actif

	Retourne ancien mode
*/
export int token_setpreproc (int mode)
{
	int tmp = preproc;
	preproc = mode;
	/* preproc_errundef (mode); */
	return (tmp);
}

/*
	Redirige la collecte des tokens dans un fichier
	Retourne 0 si ok, -1 si ne peut ouvrir
*/
export int token_open (const char *name)
{
	return filesrc_open(name);
}

/*
	Ferme le fichier ouvert avec token_open
*/
export void token_close ()
{
	filesrc_close ();
}

/*
    retourne ligne courante de lecture
*/
export int token_getnoline()
{
    return filesrc_noline;
}

/*
	Retourne un token pour tre retrait par token_get()
*/

export void token_unget (TOKEN *tok)
{
	utok = *tok;
	flg_unget = 1;
}
/*
	Saute les blancs  partir de ptl et retourne le nouveau ptl
	Si arrive au bout de la ligne, lit la suivante
	Retourne ptl pointant soit un un caractre non blanc, soit sur
	'\0' si fin de fichier
*/

char *token_skipwhite (char *ptl, char *line)
{
	while (*ptl <= ' '){
		while (isspace(*ptl)) ptl++;
		if (*ptl == '\0'){
			ptl = line;
			if (filesrc_fill (line) == -1) break;
		}
	}
	return ptl;
}

/*
	Lit un token et retourne 0 si token valide
	-1 si plus de token
*/
int token_get0 (
	TOKEN *tok,
	char **pptl,
	char *line,
	int actif,		/* !=0 si pas dans une section limine */
	int readl)		/* !=0: lit une ligne quand fin de ligne courante */
					/* ==0:	Retourne -1 a la fin de la ligne courante */
{
	int ok = 0;
	char *ptl = *pptl;
    if (flg_unget){
		*tok = utok;
        flg_unget = 0;
		return (0);
	}
	tok->longtxt = NULL;
	tok->type    = TOK_EOF;
	tok->type2.other = 0;
    while(1){
		preproc_enableact();
		ptl = str_skip (ptl);
		if (*ptl == '\0'){
			if (!readl || filesrc_fill(line)==-1){
				ok = -1;
				break;
			}
			ptl = line;
        }else if(!comment_skip (line,&ptl,filesrc_noline
			,(int)(ptl-line))){
			char carac = *ptl;
            tok->noline = filesrc_noline;
			tok->nocar  = (int)(ptl-line);
            tok->comment = comment_getlast();
			if (token_idok(*ptl)){
                ptl = token_id (tok,ptl);
				if (actif && preproc){
					if (!preproc_convert (tok,&ptl,line)){
						break;
					}else{
						/*
							Annule le token et retraite la ligne a partir
							du ptl rvis
						*/
						tok->longtxt = NULL;
						tok->type    = TOK_EOF;
						tok->type2.other = 0;
					}
				}else{
	                break;
				}
			}else if(isdigit(carac)
				|| (carac == '.' && isdigit(ptl[1]))){
				ptl = token_number (tok,ptl);
				break;
			}else if (carac == MACRO_DESACTIVE_CAR
				|| carac == MACRO_ACTIVE_CAR){
				// Dsactive/active une macro
				ptl = preproc_macroact (ptl);
            }else{
				tok->type2.other = 0;
				ptl++;
				if (carac == '#'){
					if (ptl == line + 1
						|| str_skip (line) == ptl-1){
                        tok->text[0] = '\0';
                        outsrc_prttoken (tok);  /* imprime commentaire */
						ptl = preproc_anal (line,ptl,preproc,tok);
						/* PATCH */
						actif = precond_actif;
						if (tok->type == TOK_PRAGMA) break;
					}else{
						tok->type = TOK_OTHER;
						tok->text[0] = carac;
						tok->text[1] = '\0';
						break;
					}
				}else{
					tok->text[0] = carac;
					tok->text[1] = '\0';
					switch (carac){
					case '"':
						ptl = token_string (tok,ptl-1);
						break;
					case '\'':
						ptl = token_quote (tok,ptl-1);
						break;
					case '{':
						tok->type = TOK_OPNBRACE;
						break;
					case '}':
						tok->type = TOK_CLSBRACE;
						break;
					case '(':
						tok->type = TOK_OPNPAR;
						break;
					case ')':
						tok->type = TOK_CLSPAR;
						break;
					case '[':
						tok->type = TOK_OPNBRAK;
						break;
					case ']':
						tok->type = TOK_CLSBRAK;
						break;
					case ';':
						tok->type = TOK_PTV;
						break;
					case ',':
						tok->type = TOK_VIRGULE;
						break;
					default:
						ptl = token_oper (tok,ptl-1);
					}
					break;
				}
			}
		}
	}
	*pptl = ptl;
	return (ok);
}

/*
	Lit un token et retourne 0 si token valide
	-1 si plus de token
*/
export int token_get (TOKEN *tok)
{
	int ret;
	do{
		ret = token_get0(tok,&ptl,line,precond_actif,1);
	}while (ret == 0 && !precond_actif);
	return (ret);
}

/*
	Initialise la structure de token  partir d'une chaine
*/
export void token_make (TOKEN *token,char *txt)
{
    char *tmp = ptl;
    ptl = txt;
    token_get (token);
    ptl = tmp;
}
#ifdef TEST

void main (int argc, char *argv[])
{
	TOKEN tok;
	token_init ();
	if (argc == 2){
		static char *path ="..\\h;\\turboc\\h";
		token_setpreproc(1);
		filesrc_setpath (path);
		printf ("prprocesseur actic avec path == -I%s\n",path);
	}
	while (token_get(&tok)!= -1){
		printf ("tok = %d type2 = %d :%s:\n",tok.type,tok.type2.other,
			token_txt(&tok));
	}
}

#endif

