/*
	Accumulation du text des tokens
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <etc.h>
#include "lexc.h"

#ifdef MSDOS
	#define TOKEN_TEXT  40000	/* dimension du buffer d'accumulation */
#else
	#define TOKEN_TEXT  180000	/* dimension du buffer d'accumulation */
#endif

static char *debtxt,*fintxt,*acctxt;

/*
	Initialise ou rinitialise la collecte des textes de tokens
	Les tokens (ascii) sont accumul dans un tampon. Lorsqu'on sait
	que les tokens accumuls jusqu' date ne seront plus requis
	cette fonction peut tre appel pour relacher l'espace occup.
*/

export void token_inittxt()
{
	if (debtxt == NULL) debtxt = (char*)malloc_err((unsigned)TOKEN_TEXT,1);
	fintxt = debtxt+(unsigned)TOKEN_TEXT-100;
	acctxt = debtxt;
}

/*
	Retourne une position o on peut accumuler un texte de token
	On appelle cette fonction pour obtenir un pointeur, on copie
	le texte dans le pointeur, et on appelle token_settxt() en
	passant l'adresse corrige du pointeur (saute le texte)
*/
char *token_alloctxt()
{
	if (debtxt == NULL) token_inittxt();
	assert (debtxt!=NULL);
    return (acctxt);
}
/*
	Obtient la position courante d'allocation pour le texte des token
	La fonction token_settxt() est utilis pour librer le texte de tous
	les tokens allous aprs cette fonction
*/
export char *token_getmarktxt (void)
{
	return acctxt;
}
/*
	Enregistre une nouvelle position d'allocation, librant une portion
	Le pointeur a t obtenue par token_getmarktxt().
*/
export void token_setmarktxt (char *mark)
{
	acctxt = mark;
}
/*
	Etablie la nouvelle position d'allocation des textes de token
*/

void token_settxt (char *pttxt)
{
//printf ("[%u] ",(int)(pttxt-debtxt));
//if (pttxt >= fintxt){
//	printf ("fichier %s ligne %d\n",filesrc_curfile(),filesrc_curline());
//	fflush (stdout);
//}
	assert (pttxt<fintxt);
    assert (pttxt>=acctxt);
	acctxt = pttxt;
}

/*
	Retourne le texte ascii associ au token
*/
export char *token_txt (TOKEN *tok)
{
    if (tok->longtxt != NULL) return (tok->longtxt);
	return (tok->text);
}

/*
    indique si un blanc doit etre inserer entre deux token
    Par convention tokp est celui qui precede tok1
*/
export int token_needsep (TOKEN *tok, TOKEN *tokp)
{
	int ok = 0;
    if (tok != NULL && tokp != NULL){
        if (tokp->type == TOK_OPNPAR || tok->type == TOK_CLSPAR){
            ok = 0;
        }else if (tok->type == TOK_OPNPAR){
            ok = 1;
        }else if (tokp->type == TOK_KEYWORD
			|| tokp->type == TOK_STRING
			|| tokp->type == TOK_VIRGULE){
            ok = 1;
        }else if ((tok->type == TOK_ID || tok->type == TOK_KEYWORD
                   || tok->type == TOK_STAR)
            && (tokp->type == TOK_ID)){
            ok = 1;
        }
    }
    return (ok);
}
/*
	Imprime le texte d'un token sur stdout
*/
export int token_print (TOKEN *tok)
{
    int nbout=0;
    if (tok->type == TOK_ID || tok->type == TOK_KEYWORD) nbout = printf (" ");
    nbout += printf ("%s",token_txt(tok));
    return (nbout);
}

/*
	retourne 1 si token identique, 0 sinon
*/
int token_egaux (TOKEN *tok1, TOKEN *tok2)
{
	if (tok1->type == tok2->type 
		&& tok1->type2.other == tok2->type2.other
		&& strcmp(token_txt(tok1),token_txt(tok2))==0){
		return (1);
	}
    return (0);
}
