#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <etc.h>
#include "permut.h"
#include "clsinfo.h"
#include "nadoc.m"

#define CLSFORM_VERSION	1
#define CLSFORM_RELEASE	3

static void showver()
{
	fprintf (stderr,"clsform %d.%d (linuxconf-tools %d.%d)\n"
		,CLSFORM_VERSION,CLSFORM_RELEASE
		,TOOLS_VERSION,TOOLS_RELEASE);
}

static void near usage(void)
{
	showver();
	fprintf (stderr,"%s\n",MSG_B(I_CLSFORM
		,"clsform [options] fichier_proto ...\n"
		 "\n"
		 "       Formatte la hierarchie des classes extraite\n"
		 "       par proto (option -q).\n"
		 "\n"
		 "  -cC  Extrait la liste des fonctions membres de la classe C\n"
		 "       incluant les fonctions membres des classes de base.\n"
		 "       -c+C extrait les fonction interne de la classe.\n"
		 "  -fF  Lit le fichier F contenant des prototypes de fonction\n"
		 "       membres. Ce fichier est gnralement produit avec l'option\n"
		 "       -c+d de la commande proto.\n"
		 "       L'option -f peut tre rpte plusieurs fois.\n"
		 "  -g   Produit du sgml.\n"
		 "  -oF  Produit le rsultat dans le fichier F.\n"
		 "  -t   Produit l'arbre des classes.\n"

		,"clsform [options] class_file ...\n"
		 "\n"
		 "       Format the class hierarchy extracted by proto.\n"
		 "       (See the -q option)\n"
		 "\n"
		 "    -g Produce sgml output.\n"));
}


/*
	Gnre le code de dclaration d'une classe.
	Ne fait rien si c'est dj fait.
*/
PUBLIC void CLSFORM::gentest0 (
	CLS_INFO *info,
	FILE *fout)
{
	if (info->nbprint == 0){
		char buffer[500];
		int premder = 1;
		const char *nom = info->nom;
		char *pt = buffer +  sprintf(buffer,"class %s",nom);
		int nbder = ders.getnb();
		int i;
		for (i=0; i<nbder; i++){
			CLS_DERIVE *ptder = ders.getitem(i);
			if (ptder->info == info && ptder->base->nom[0] != '-'){
				CLS_INFO *base = ptder->base;
				gentest0 (base,fout);
				*pt++ = premder ? ':' : ',';
				premder = 0;
				pt += sprintf (pt,"%s %s",ptder->opt,base->nom);
			}
		}
		info->nbprint++;
		fprintf (fout,"%s{\npublic:\n",buffer);
		fprintf (fout,"\tchar data_%s[%d];\n",nom,i);
		if (premder){
			fprintf (fout,"\tchar *nomcls;\n\tvoid printbase();\n");
		}
		fprintf (fout,"\t%s();\n\tvirtual void print();\n};\n",nom);
		fprintf (fout,"%s::%s(){\n\tnomcls=\"%s\";\n}\n",nom,nom,nom);
		if (premder){
			fprintf (fout,"void %s::printbase(){\n\tprint();\n}\n",nom);
		}
		fprintf (fout,"void %s::print(){\n",nom);
		fprintf (fout,"\tprintf (\"%s %s\\n\",nomcls);\n",nom,nom);
		fprintf (fout,"}\n");
	}
}
/*
	Gnre un fichier permettant de tester des cas de drivation
	extrait de programme rel.
*/
PUBLIC int CLSFORM::gentest (FILE *fout)
{
	infos.getitem(0)->nbprint = 1;
	int nbinfo = infos.getnb();
	for (int i=1; i<nbinfo; i++){
		infos.getitem(i)->nbprint = 0;
	}
	for (int i=1; i<nbinfo; i++){
		gentest0 (infos.getitem(i),fout);
	}
	/* Gnre le programme principal */
	fprintf (fout,"int main (int, char *[])\n{\n");
	for (int i=1; i<nbinfo; i++){
		CLS_INFO *ptinfo = infos.getitem(i);
		fprintf (fout,"\t%s v_%s;\n",ptinfo->nom,ptinfo->nom);
		fprintf (fout,"\tv_%s.printbase();\n",ptinfo->nom);
	}
	fprintf (fout,"\treturn 0;\n}\n");
	return 0;
}

int main (int argc, char *argv[])
{
	etc_loadmsg();
	int ret = -1;
	ARGP_MULTI argp[26];
	argc = anlparm_multi(argc,argv,argp,"cf*gotv");
	if (argp['v'-'a'].ptr != NULL){
		showver();
	}else if (argc < 2){
		usage();
	}else{
		bool sgml = argp['g'-'a'].ptr != NULL;
		CLSFORM ccls;
		ret = 0;
		for (int i=1; i<argc; i++){
			if (ccls.read (argv[i]) == -1){
				ret = -1;
			}
		}
		ccls.unknown ();
		if (ret == 0){
			FILE *fout = stdout;
			if (argp['o'-'a'].ptr != NULL){
				fout = fopen_err (argp['o'-'a'].ptr,"w",1);
			}
			if (argp['t'-'a'].ptr != NULL){
				ret = ccls.print (fout,sgml);
			}
			if (argp['c'-'a'].ptr != NULL){
				const char *cls = argp['c'-'a'].ptr;
				if (cls != NULL){
					ARGP_MULTI *arg = &argp['f'-'a'];
					NAP_LIST lst;
					for (int i=0; i<arg->nbptr; i++){
						char *ptr = arg->ptrs[i];
						lst.parse (ptr);
					}
					bool vue_interne = false;
					if (cls[0] == '+'){
						vue_interne = true;
						cls++;
					}
					ccls.flatfunc (cls,lst,vue_interne,sgml,fout);
				}
			}
		}
	}
	return ret;
}

