#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <etc.h>
#include <misc.h>
#include "pagin.h"
#include "permut.h"
#include "nadoc.m"

PUBLIC PAGIN_SGML::PAGIN_SGML()
{
	fout = NULL;
	sgml_aware = false;
}

PUBLIC PAGIN_SGML::~PAGIN_SGML()
{
	close();
}
/*
	Ouvre un fichier d'output avec pagination
	Retourne -1 si erreur
*/
PUBLIC int PAGIN_SGML::openerr (
	const char *fname,
	const char *title,
	const char *author,
	int quit)
{
	fout = fopen_err (fname,"w",quit);
	format = PAG_FORMAT_PARA;
	if (title == NULL) title = fname;
	if (author == NULL){
		author = getenv ("USER");
		if (author == NULL) author = "------------------";
	}
	::fprintf (fout,"<!doctype linuxdoc system>\n"
			"<article>\n"
			"<title>%s</title>\n"
			"<author>%s</author>\n"
			,title,author);
	return 0;
}

/*
	Termine output dans un fichier
*/
PUBLIC void PAGIN_SGML::close ()
{
	if (fout != NULL){
		::fprintf (fout,"</article>\n");

		fclose (fout);
		fout = NULL;
	}
}
/*
	Controle le mode de formattage des paragraphe.
	Retourne la valeur courante.
*/
PUBLIC int PAGIN_SGML::set_format (int _format)
{
	int ret = format;
	if (_format != format){
		char *cmd = "";
		if (format == PAG_FORMAT_NONE){
			cmd = "</verb></tscreen>\n";
		}else if (format == PAG_FORMAT_QUOT){
			cmd = "</verb></tscreen>\n";
		}
		::fprintf (fout,"%s",cmd);
		PAGIN_V::set_format (_format);
		cmd = "";
		if (format == PAG_FORMAT_NONE){
			cmd = "<tscreen><verb>\n";
		}else if (format == PAG_FORMAT_QUOT){
			cmd = "<tscreen><verb>\n";
		}
		::fprintf (fout,"%s",cmd);
	}
	return ret;
}

/*
	Gnre un saut de page et ajuste
*/
PUBLIC void PAGIN_SGML::form ()
{
}

static void paging_fputc (char car, FILE *fout, bool sgml_aware)
{
	if (car == '&'){
		::fputs ("&amp;",fout);
	}else if (car == '$'){
		::fputs ("&dollar;",fout);
	}else if (car == '#'){
		::fputs ("&num;",fout);
	}else if (car == '%'){
		::fputs ("&percnt;",fout);
	}else if (car == '~'){
		::fputs ("&tilde;",fout);
	}else if (car == '"'){
		::fputs ("&dquot;",fout);
	}else if (!sgml_aware){
		if (car == '<'){
			::fputs ("&lt;",fout);
		}else if (car == '>'){
			::fputs ("&gt;",fout);
		}else{
			fputc (car,fout);
		}
	}else{
		fputc (car,fout);
	}
}

static void paging_url (
	const char *file,
	const char *entry,
	FILE *fout)
{
	::fprintf (fout,"<url url=\"%s#---%s---\" name=\"%s\">"
		,file,entry,entry);
}

PUBLIC void PAGIN_SGML::fputs (const char *buf, FILE *fout)
{
	if (format != PAG_FORMAT_PARA){
		while (*buf != '\0') paging_fputc (*buf++,fout,sgml_aware);
	}else{
		crosstbl_filter (buf,fout,paging_fputc,paging_url,sgml_aware);
	}
}
PUBLIC void PAGIN_SGML::printf (
	const char *ctl,
	...)
{
	if (format == PAG_FORMAT_QUOT) fputs ("\t",fout);
	va_list list;
	va_start (list,ctl);
	char buf[5000];
	vsnprintf (buf,sizeof(buf)-1,ctl,list);
	fputs (buf,fout);
	va_end (list);
}
/*
	fprintf avec traduction.
*/
PUBLIC void PAGIN_SGML::fprintf (
	FILE *f,
	const char *ctl,
	...)
{
	va_list list;
	va_start (list,ctl);
	char buf[5000];
	vsprintf (buf,ctl,list);
	va_end (list);
	fputs (buf,f);
}
/*
	Saute une ligne
*/
PUBLIC void PAGIN_SGML::nl ()
{
	//fputs ("<P>\n",fout);
}
/*
	Vrifie s'il y a au moins N lignes disponible jusqu' la fin de la page
	Si pas assez, gnre un saut de page.

	Le nombre de lignes est automatiquement ajout. L'appelant devra
	affich lui-mme sans utiliser les fonctions de pagin. Le compte
	sera fauss sinon
*/
PUBLIC void PAGIN_SGML::check (int)
{
}

/*
	Gnre un paragraphe d'un coup en formattant ou pas.
	En mode SGML, on ne s'occupe pas de ca. C'est set_format qui fait
	tout.
*/
PUBLIC void PAGIN_SGML::format_paragraphe (char *buf)
{
	if (format) ::fputs ("<P>\n",fout);
	fputs (buf,fout);
	fputs ("\n",fout);
}
/*
	Gnre le titre d'une section.
*/
PUBLIC void PAGIN_SGML::format_titre (
	int niveau,				// Niveau de numrotation
	const char *secnum,		// Numrotation de la section
	char *buf)				// Titre de la section
{
	int old_format = format;
	format = PAG_FORMAT_QUOT;
	if (niveau == 0){
		::fputs ("<sect>",fout);
		printf ("%s\n",buf);
		::fputs ("<p>\n",fout);
	}else{
		::fprintf (fout,"<sect%d>",niveau);
		printf ("%s\n",buf);
		::fputs ("<p>\n",fout);
	}
	format = old_format;
}


/*
	Gnre l'entte d'une spcification.
*/
PUBLIC void PAGIN_SGML::format_spec (
	const char *filespec,	// Nom du fichier qui contient cette spec
	const char *buf)		// Titre de la spec (cl de classement)
{
	nl();
	::fputs ("<p>\n",fout);
	printf ("%s (%s)\n\n",buf,filespec);
#if 0
	char cle[300];
	pagin_spec2hname (filespec,buf,cle);
	fprintf (fout,"<A NAME = \"%s\">\n",cle);
	fprintf (fout,"  %s<BR>\n",filespec);
	fprintf (fout,"  <STRONG>%s</STRONG>\n",buf);
	fprintf (fout,"</A>\n");
	nl();
#endif
}
/*
	Gnre l'entte d'une fonction.
*/
PUBLIC void PAGIN_SGML::format_fct (
	const char *fct,	// Nom du fichier qui contient cette spec
	int premiere)		// Premiere fois qu'on parle de cette fonction
{
	::fputs ("<p>\n",fout);
	if (premiere){
		::fprintf (fout,"<label id=\"---%s---\"><bf>%s</bf>()<p>\n",fct,fct);
	}else{
		::fprintf (fout,"<ref id=\"---%s---\" name=\"%s()\"><p>\n",fct,fct);
	}
	nl();
}
/*
	Gnre un label
*/
PUBLIC void PAGIN_SGML::format_ref (
	const char *ref)	// Rfrence  encoder
{
	::fprintf (fout,"<label id=\"---%s---\">",ref);
}
/*
	Dbute une sous-section (sous liste numr)
*/
PUBLIC void PAGIN_SGML::soussection()
{
}
/*
	Termine une sous-section (sous liste numr)
*/
PUBLIC void PAGIN_SGML::finsous()
{
}
/*
	Controle le formattage de la page index
*/
PUBLIC PAGIN_SGML_IDX::PAGIN_SGML_IDX ()
{
}
PUBLIC PAGIN_SGML_IDX::~PAGIN_SGML_IDX ()
{
}
/*
	Dbute une sous-section (sous liste numr)
*/
PUBLIC void PAGIN_SGML_IDX::soussection()
{
}
/*
	Termine une sous-section (sous liste numr)
*/
PUBLIC void PAGIN_SGML_IDX::finsous()
{
}
/*
	Gnre le titre d'une section.
*/
PUBLIC void PAGIN_SGML_IDX::format_titre (
	int ,				// Niveau de numrotation
	const char *,		// Numrotation de la section
	char *)				// Titre de la section
{
}
/*
	Gnre l'entte d'une spcification.
*/
PUBLIC void PAGIN_SGML_IDX::format_spec (
	const char *,	// Nom du fichier qui contient cette spec
	const char *)		// Titre de la spec (cl de classement)
{
}
/*
	Gnre l'entte d'une fonction.
*/
PUBLIC void PAGIN_SGML_IDX::format_fct (
	const char *,	// Nom du fichier qui contient cette spec
	int)		// Premiere fois qu'on parle de cette fonction
{
}

/*
	Ouvre un fichier d'output pour l'index.
	Retourne -1 si erreur
*/
PUBLIC int PAGIN_SGML_IDX::openerr (
	const char *,
	int quit)
{
	return 0;
}


