#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <tool.h>
#include <etc.h>
#include "pagin.h"
#include "permut.h"


PUBLIC PAGIN_HTML::PAGIN_HTML()
{
	fout = NULL;
	basename = NULL;
	title = NULL;
}

PUBLIC PAGIN_HTML::~PAGIN_HTML()
{
	close();
	free (basename);
	free (title);
}
/*
	Ouvre un fichier d'output avec pagination
	Retourne -1 si erreur
*/
PUBLIC int PAGIN_HTML::openerr (
	const char *_basename,		// Nom de base servant a composer les
								// noms des diffrents fichiers html
	int quit)
{
	basename = strdup_err (_basename,quit);
	format = PAG_FORMAT_PARA;
	sequence = 0;
	return 0;
}

/*
	Termine output dans un fichier
*/
PUBLIC void PAGIN_HTML::close ()
{
	if (fout != NULL){
		fclose (fout);
		fout = NULL;
	}
}
/*
	Controle le mode de formattage des paragraphe.
	Retourne la valeur courante.
*/
PUBLIC int PAGIN_HTML::set_format (int _format)
{
	int ret = format;
	if (_format != format){
		char *cmd = "";
		if (format == PAG_FORMAT_NONE){
			cmd = "</PRE>\n";
		}else if (format == PAG_FORMAT_QUOT){
			cmd = "</PRE>\n";
		}
		fprintf (fout,"%s",cmd);
		PAGIN_V::set_format (_format);
		if (format == PAG_FORMAT_NONE){
			cmd = "<PRE>\n";
		}else if (format == PAG_FORMAT_QUOT){
			cmd = "<PRE>\n";
		}
		fprintf (fout,"%s",cmd);
	}
	return ret;
}

/*
	Gnre un saut de page et ajuste
*/
PUBLIC void PAGIN_HTML::form ()
{
	fprintf (fout,"<saut de page>\n");
}
/*
	Traduit une chaine en ISO et ajoute dans un fichier
*/
PUBLIC void PAGIN_HTML::fputs (const char *buf, FILE *fout)
{
	::fputs (buf,fout);
}
/*
	Affiche une seule ligne en dcidant si on saute de page
*/
PUBLIC void PAGIN_HTML::printf (
	const char *ctl,
	...)
{
	if (format == PAG_FORMAT_QUOT) fputs ("\t",fout);
	va_list list;
	va_start (list,ctl);
	char buf[5000];
	vsprintf (buf,ctl,list);
	fputs (buf,fout);
	va_end (list);
}
/*
	fprintf avec traduction.
*/
PUBLIC void PAGIN_HTML::fprintf (
	FILE *f,
	const char *ctl,
	...)
{
	va_list list;
	va_start (list,ctl);
	char buf[5000];
	vsprintf (buf,ctl,list);
	va_end (list);
	fputs (buf,f);
}
/*
	Saute une ligne
*/
PUBLIC void PAGIN_HTML::nl ()
{
	fputs ("<P>\n",fout);
}
/*
	Vrifie s'il y a au moins N lignes disponible jusqu' la fin de la page
	Si pas assez, gnre un saut de page.

	Le nombre de lignes est automatiquement ajout. L'appelant devra
	affich lui-mme sans utiliser les fonctions de pagin. Le compte
	sera fauss sinon
*/
PUBLIC void PAGIN_HTML::check (int)
{
}

/*
	Gnre un paragraphe d'un coup en formattant ou pas.
	En mode HTML, on ne s'occupe pas de ca. C'est set_format qui fait
	tout.
*/
PUBLIC void PAGIN_HTML::format_paragraphe (char *buf)
{
	if (format) fputs ("<P>\n",fout);
	fputs (buf,fout);
	fputs ("\n",fout);
}
/*
	Gnre le titre d'une section.
*/
PUBLIC void PAGIN_HTML::format_titre (
	int niveau,				// Niveau de numrotation
	const char *secnum,		// Numrotation de la section
	char *buf)				// Titre de la section
{
	niveau++;
	if (niveau == 1){
		sequence++;
		if (sequence == 1) title = strdup_err (buf,1);
		if (fout != NULL) fclose (fout);
		char path[MAXSIZ_PATH];
		sprintf (path,"%s.%d.html",basename,sequence);
		fout = fopen_err (path,"w",1);
		fprintf (fout,"<TITLE>%s</TITLE>\n",title);
	}
	fprintf (fout,"<A NAME = \"%s\">\n",secnum);
	fprintf (fout,"<H%d>%s</H%d>\n",niveau,buf,niveau);
	fprintf (fout,"</A>\n");
}

static void pagin_spec2hname (
	const char *filespec,
	const char *,
	char *cle)
{
	/* #Spcification: spcification / rfrence hypertexte
		On compose une rfrence hypertexte vers une spec comme suit:

		On utilise simplement la rfrence source [fichier,ligne]
		indiquant d'ou provient cette spec. On est assur d'une cl unique.
	*/
	strcpy (cle,filespec);
}
/*
	Gnre l'entte d'une spcification.
*/
PUBLIC void PAGIN_HTML::format_spec (
	const char *filespec,	// Nom du fichier qui contient cette spec
	const char *buf)		// Titre de la spec (cl de classement)
{
	nl();
	char cle[300];
	pagin_spec2hname (filespec,buf,cle);
	fprintf (fout,"<A NAME = \"%s\">\n",cle);
	fprintf (fout,"  %s<BR>\n",filespec);
	fprintf (fout,"  <STRONG>%s</STRONG>\n",buf);
	fprintf (fout,"</A>\n");
	nl();
}
/*
	Gnre l'entte d'une fonction.
*/
PUBLIC void PAGIN_HTML::format_fct (
	const char *fct,	// Nom du fichier qui contient cette spec
	int premiere)		// Premiere fois qu'on parle de cette fonction
{
	nl();
	if (premiere){
		fprintf (fout,"<A NAME = \"%s()\">\n",fct);
		fprintf (fout,"<STRONG>%s()</STRONG>\n",fct);
		fprintf (fout,"</A>\n");
	}else{
		fprintf (fout,"<STRONG>%s()</STRONG>\n",fct);
	}
	nl();
}

/*
	Gnre un label
*/
PUBLIC void PAGIN_HTML::format_ref (
	const char *ref)	// Rfrence  encoder
{
	::fprintf (fout,"<A NAME = \"---%s---\"></A>",ref);
}

/*
	Dbute une sous-section (sous liste numr)
*/
PUBLIC void PAGIN_HTML::soussection()
{
}
/*
	Termine une sous-section (sous liste numr)
*/
PUBLIC void PAGIN_HTML::finsous()
{
}
/*
	Controle le formattage de la page index
*/
PUBLIC PAGIN_HTML_IDX::PAGIN_HTML_IDX ()
{
	curniv = 0;
}
PUBLIC PAGIN_HTML_IDX::~PAGIN_HTML_IDX ()
{
	fclose (mainfout);
}
/*
	Dbute une sous-section (sous liste numr)
*/
PUBLIC void PAGIN_HTML_IDX::soussection()
{
	fprintf (fout,"<OL>\n");
}
/*
	Termine une sous-section (sous liste numr)
*/
PUBLIC void PAGIN_HTML_IDX::finsous()
{
	fprintf (fout,"</OL>\n");
}
/*
	Gnre le titre d'une section.
*/
PUBLIC void PAGIN_HTML_IDX::format_titre (
	int niveau,				// Niveau de numrotation
	const char *secnum,		// Numrotation de la section
	char *buf)				// Titre de la section
{
	niveau++;
	char path[MAXSIZ_PATH];
	if (niveau == 1){
		sequence++;
		if (sequence == 1){
			/* #Spcification: html / index des documents
				Pour chaque document html produit par nadoc, il y a un
				fichier "document".index qui est produit. Il est plac dans
				le mme rpertoire.

				Le script Unix suivant permet de fabriquer un document
				d'introduction  tout les documents html produit par nadoc.

				#
				#!/bin/sh
				INDEX=index.html
				echo '<DL>' > $INDEX
				for rep in *
				do
					if [ -f $rep/$rep.index ] ; then
						cat $rep/$rep.index >> $INDEX
					fi
				done
				echo '</DL>' >> $INDEX
				#
			*/
			sprintf (path,"%s.index",basename);
			FILE *f = fopen_err (path,"w",1);
			sprintf (path,"%s.i.html",rel_path);
			fprintf (f,"<A HREF=\"%s\">\n",path);
			//fprintf (f,"<DT>%s\n",rel_path);
			fprintf (f,"<LI>%s\n",buf);
			fprintf (f,"</A>\n");
			fclose (f);			
			fprintf (fout,"<TITLE>%s</TITLE>\n",buf);
		}
		sprintf (path,"%s.i.%d.html",rel_path,sequence);
		fprintf (mainfout,"<A HREF=\"%s#%s\">\n",path,secnum);
		fprintf (mainfout,"<LI>%s\n",buf);
		fprintf (mainfout,"</A>\n");
		close();
		sprintf (path,"%s.i.%d.html",basename,sequence);
		fout = fopen_err (path,"w",1);
	}
	/* #Spcification: html / page index
		La page index est simplement une srie de lien vers les
		pages du document. Ces liens sont prsents via une srie
		d'numrations numrotes.
	*/
	curniv = niveau;
	fprintf (fout,"<A NAME = \"%s\">\n",secnum);
	sprintf (path,"%s.%d.html",rel_path,sequence);
	fprintf (fout,"<A HREF=\"%s#%s\">\n",path,secnum);
	fprintf (fout,"<LI>%s\n",buf);
	fprintf (fout,"</A>\n");
}
/*
	Gnre l'entte d'une spcification.
*/
PUBLIC void PAGIN_HTML_IDX::format_spec (
	const char *filespec,	// Nom du fichier qui contient cette spec
	const char *buf)		// Titre de la spec (cl de classement)
{
	char cle[300];
	pagin_spec2hname (filespec,buf,cle);
	char path[MAXSIZ_PATH];
	sprintf (path,"%s.%d.html",rel_path,sequence);
	fprintf (fout,"<A HREF=\"%s#%s\">\n",path,cle);
	fprintf (fout,"<BR>%s\n",buf);
	fprintf (fout,"</A>\n");
}
/*
	Gnre l'entte d'une fonction.
*/
PUBLIC void PAGIN_HTML_IDX::format_fct (
	const char *fct,	// Nom du fichier qui contient cette spec
	int premiere)		// Premiere fois qu'on parle de cette fonction
{
	if (premiere){
		char path[MAXSIZ_PATH];
		sprintf (path,"%s.%d.html",rel_path,sequence);
		fprintf (fout,"<A HREF=\"%s#%s()\">\n",path,fct);
		fprintf (fout,"<BR>%s()\n",fct);
		fprintf (fout,"</A>\n");
	}
}

/*
	Ouvre un fichier d'output pour l'index.
	Retourne -1 si erreur
*/
PUBLIC int PAGIN_HTML_IDX::openerr (
	const char *_basename,		// Nom de base servant a composer les
								// noms des diffrents fichiers html
	const char *_rel_path,
	int quit)
{
	basename = strdup_err (_basename,1);
	rel_path = strdup_err (_rel_path,1);
	format = 1;
	sequence = 0;
	char path[MAXSIZ_PATH];
	sprintf (path,"%s.%d.html",_basename,sequence);
	mainfout = fopen_err (path,"w",quit);
	return mainfout != NULL ? 0 : -1;
}

