#include <string.h>
#include <dialog.h>
#include "projet.h"
#include "projetx.m"
#include "fusion.h"

struct TBREV {
	VERSION_ONE *tb[400];
	int nb;
};
/*
	Ajoute une rvision dans un tableau.
	Insre en ordre de rvision.

	Si la rvision est dj l, elle est retire.

	Retourne le nouveau nombre de rvision dans la table.
*/
static int near bfusion_addrev (
	TBREV &revs,
	VERSION_ONE *newone,
	int &change,	// Indique quel item de la table a change
	int &efface,	// Indique que l'item a t effac
	int doefface)	// Efface si deja la ou conserve ?
{
	int noins = 0;
	change = -1;
	efface = 0;
	const REVISION *newrev = newone->getrev();
	int i;
	for (i=0; i<revs.nb; i++){
		int cmp = newrev->cmp(revs.tb[i]->getrev());
		if (cmp==0){
			noins = 1;
			if (doefface){
				revs.nb--;
				for (; i<revs.nb; i++) revs.tb[i] = revs.tb[i+1];
				change = i;
				efface = 1;
			}
			break;
		}else if (cmp < 0){
			// C'est ici qu'on doit l'insrer
			break;
		}
	}
	if (!noins){
		if (revs.nb == NB_ELM(revs.tb)){
			xconf_error (MSG_U(E_MAXREVSEL
				,"Nombre maximum slectionnable: %d")
				,NB_ELM(revs.tb));
		}else{
			for (int r=revs.nb; r>i; r--) revs.tb[r] = revs.tb[r-1];
			revs.tb[i] = newone;
			change = i;
			revs.nb++;
		}
	}
	return revs.nb;
}

/*
	Programme une grille d'dition dans un page.
	Permet d'associe des rvision (revs.tb) a des rfrences (refs.tb).
	(une rfrence par rvision).
*/
static void near bfusion_setgrille (
	PAGE_EDIT &page,
	TBREV &revs,		// Lignes
	TBREV &refs,		// Colonnes
	char rev2ref[])
{
	char buf[100];
	if (refs.nb == 0){
		buf[0] = ' ';
		buf[1] = '\0';
	}else{
		char *ptbuf = buf;
		for (int i=0; i<refs.nb; i++){
			refs.tb[i]->getrev()->format (ptbuf);
			strcat (ptbuf,"  ");
			ptbuf += strlen(ptbuf);
		}
	}
	int lenbuf = strlen(buf);
	page.zap();
	page_setuptitre (&page,MSGTIT_GRILLE);
	PAGE_EDIT *sp1 = new PAGE_EDIT ("");
	sp1->setboxtype (WINBOX_NO);
	page_setupfixestr0 (sp1,-1,"",MAXSIZ_REVISION);
	page_setupfixestr0 (sp1,-1,buf,lenbuf);
	sp1->linedesign();
	page.setup (-1,sp1,1);
	PAGE_EDIT *sp2 = new PAGE_EDIT ("");
	sp2->setboxtype (WINBOX_NO);
	sp2->setscrollv (9);
	for (int i=0; i<revs.nb; i++){
		char revstr[MAXSIZ_REVISION];
		revs.tb[i]->getrev()->format(revstr);
		page_setupfixestr0 (sp2,-1,revstr,MAXSIZ_REVISION);
		if (refs.nb > 0){
			page_setupenumvc (sp2,-1,&rev2ref[i],refs.nb,buf,0);
		}else{
			page_setupfixestr0 (sp2,-1,"",lenbuf);
		}
	}
	// S'assure que la page sp2 n'est jamais vide et de largeur
	// identique a sp1. On ajoute des champs bidon a la fin.
	page_setupfixestr0 (sp2,-1,"",MAXSIZ_REVISION);
	page_setupfixestr0 (sp2,-1,"",lenbuf);
	sp2->linedesign0(2);
	page.setup (-1,sp2,1);
	page.coldesign();
}
	

/*
	Lit un fichier memo et preprogramme une partie de la grille de slection
*/
static void bfusion_readmemo (
	USERINFO *user,
	VERSION_DAT &version,
	TBREV &revs,
	TBREV &refs,
	char rev2ref[])
{
	/* #Spcification: fusion / composition / fichier preset
		Avec le nombre de programmeur qui augmente, la slection des
		composantes pour une intgration devient plus hasardeuse.
		Spcialement dans le cas o on doit intgrer des rvisions
		relative  une rfrence relativement vieille. On a souvent
		le scnario suivant.

		"Salut! Je suis le programmeur X et je viens de livrer 4.10.40.2.
		 4.10.40.1 avait t intgrer dans 4.11. Pourrait tu t'assurer
		 que ces corrections seront fusionnes dans 4.30. Merci!"

		Voici une solution qui permet d'aider l'intgrateur. Lors de
		l'intgration de 4.30, projetp consulte le fichier
		/kit/build/_430.itg. Si prsent, il s'en servira pour initialiser
		la grille de slection de l'intgration. Dans le cas d'une fusion
		dans l'espace usager, le meme fichier sera consult mais dans
		le HOME de l'usager.

		Ce fichier est en format texte. Il contient une paire de revision
		par ligne. Les rvisions sont en format "lisible" (x.x.x.x). La
		premire est la rvision a intgrer. La seconde est la rfrence
		a utiliser.

		Dans l'exemple prcdant, l'intgrateur inscrirait simplement

		4.10.40.2	4.10.40.1

		Ce fichier supporte les commentaires  la makefile avec le #.
	*/
	char path[MAXSIZ_PATH];
	user->makusrpath ("",path);
	file_chgext (path,path,".itg");
	// Ce fichier est optionnel. Pas d'erreur si absent
	FILE *fin = fopen (path,"r");
	if (fin != NULL){
		char buf[200];
		struct {
			VERSION_ONE *rev;
			VERSION_ONE *ref;
		}tb[200];
		int nb = 0;
		while (fgets_strip(buf,sizeof(buf)-1,fin,'\\','#',NULL)!=NULL){
			char *ptrev = str_skip (buf);
			char *pt = str_skipword (ptrev);
			if (*pt != '\0'){
				*pt++ = '\0';
				pt = str_skip (pt);
				REVISION rev (ptrev);
				REVISION ref (pt);
				VERSION_ONE *onerev = version.get(&rev);
				if (onerev == NULL){
					error_showdef (MSGERR_IVLDREVISION,ptrev);
				}else{
					VERSION_ONE *oneref = version.get(&ref);
					if (oneref == NULL){
						error_showdef (MSGERR_IVLDREVISION,pt);
					}else{
						tb[nb].rev = onerev;
						tb[nb].ref = oneref;
						nb++;
					}
				}
			}
		}
		fclose (fin);
		int change,efface;
		for (int i=0; i<nb; i++){
			bfusion_addrev (refs,tb[i].ref,change,efface,0);
		}
		for (i=0; i<nb; i++) bfusion_addrev (revs,tb[i].rev,change,efface,0);
		// Mise a jour de la grille
		for (i=0; i<revs.nb; i++){
			for (int j=0; j<refs.nb; j++){
				if (tb[i].ref == refs.tb[j]){
					rev2ref[i] = (char)j;
					break;
				}
			}
		}
	}
	
}

/*
	Slectionne la combinaison de rvision qui servira  la fusion.
	Slectionne
			-Les rvisions a fusionner
			-Les rfrences a utiliser
			-La rvision hote

	Retourne le nombre d'lments dans tbpaire[] ou -1 si abandon
	tbpaire contient des pointeurs extrait de l'objet version.
*/
int bfusion_compose (
	USERINFO *user,
	VERSION_PAIRE tbpaires[],
	VERSION_DAT &version)
{
	/* #Spcification: fusion / principe
		Pour raliser une fusion, on doit slectionner un certain
		nombre de rvision ainsi que des rfrences correspondantes
		et prsenter tous ca dans le contexte d'une rvision hote.

		Le rsultat de la fusion (intgration) sera

		hote
			+ (rev1 - ref1)
			+ (rev2 - ref2)
			+ (revn - refn)

		rev1 - ref1 indique que seule les changements entre rev1 et
		ref1 seront ajoute (sous le controle de l'intgrateur)  la
		rvision hote
	*/
	/* #Spcification: fusion / dialogue de composition
		Pour slectionner les rvisions qui serviront  la fusion
		on a un dialogue spar en deux sections verticales. Au dessus
		il y a une section de statut.

		La section de gauche contient la liste des rvisions disponible
		dans un PAGE_PICK. Sous le PAGE_PICK se retrouve 3 boutons.
		Le bouton opre sur la rvision slectionne dans le PAGE_PICK.

		hote: Cette rvision deviendra celle qui agira comme rvision
			  par dfaut. Quand on slectionne une rvision avec ce bouton
			  la ligne "rvision hote" dans la section "statut" est
			  modifie.
		addrev:	Ajoute cette rvision  la liste des rvisions a fusionner
		addref: Ajoute cette rvision  la liste des rfrences

		La section de droite est en fait une grille

					4.14	4.14.26.2	4.16	4.17
		4.17.2.1								 X
		4.17.3.2								 X
		4.17.4.1								 X
		4.16.9.1						 X
		4.14.15.1	 X
		4.14.26.4			 X


		A chaque rvision slectionn avec le bouton addrev, une ligne
		est insr.

		A chaque rvision slectionn avec le bouton addref, un colonne
		est insr.

		Si on slectionne une rvision dj slectionn, ca limine
		la slection (enlve une ligne ou une colonne)
	*/
	PAGE_EDIT page("fusion_compose");
	page_setuptitre (&page,MSGTIT_COMPOSE);
	projet_setupinfo (&page,user);
	unsigned select = 0;
	PAGE_PICK pick("pick_rev",10,&select);
	page_setuptitre (&pick,MSGTIT_VERSIONS);
	version.setuppick (pick);
	PAGE_EDIT combine ("fusion_combine");
	PAGE_EDIT sp("");
	sp.setup (-1,&pick,0);
	sp.setup (-1,&combine,0);
	/* #Spcification: fusion / dialogue de composition / double click
		Un double click dans la liste des rvisions est quivalent
		au bouton "add rev". La touche enter aussi.
	*/
	bouton_setupuser (&sp,MSGBUT_ADDREV,K_ENTER,PAGE_VAL_USER2);
	bouton_setupuser (&sp,MSGBUT_ADDREF,K_NULL,PAGE_VAL_USER3);
	sp.setboxtype (WINBOX_NO);
	page.setup (-1,&sp,0);
	bouton_setupstd (&page,"cancel ok");
	TBREV revs;
	revs.nb = 0;
	TBREV refs;
	refs.nb = 0;
	char rev2ref[NB_ELM(refs.tb)];	// associe une rvision  une rfrence
	memset (rev2ref,0,sizeof(rev2ref));
	bfusion_readmemo (user,version,revs,refs,rev2ref);
	int ret = -1;
	bfusion_setgrille(combine,revs,refs,rev2ref);
	sp.linedesign();
	page.coldesign();
	page.draw();
	while (1){
		PAGE_VAL val = page.edit (mskwin,cmd);
		if (cmd->is_load){
			break;
		}else{
			page.save();
			VERSION_ONE *one = version.get (select);
			if (val == PAGE_VAL_ABORT){
				break;
			}else if (val == PAGE_VAL_OK){
				if (revs.nb == 0 || refs.nb == 0){
					error_showdef (MSGERR_NOREV);
				}else{
					VERSION_PAIRE *ptp = tbpaires;
					for (int i=0; i<revs.nb; i++, ptp++){
						ptp->var = revs.tb[i];
						// Attention, un champs enum doit tre interprt
						// diffremment quand il y a un seul choix.
						// On s'en fout, parce que s'il y a une seule
						// rfrence elle est slectionn pour toutes
						// les rvisions.
						// Quand il y a plusieurs rfrences, le champs
						// agit comme un slecteur et non comme un booleen.
						ptp->ref = refs.nb == 1
							? refs.tb[0] : refs.tb[rev2ref[i]];
					}
					ret = revs.nb;
					break;
				}
			}else{
				int change_ref = -1;
				int change_rev = -1;
				if (val == PAGE_VAL_USER2){
					int efface;
					bfusion_addrev (revs,one,change_rev,efface,1);
					if (change_rev >=0){
						// On a ajout ou retirer une ligne
						// Il faut donc synchroniser rev2rev
						if (efface){
							for (int i=change_rev; i<revs.nb; i++){
								rev2ref[i] = rev2ref[i+1];
							}
						}else{
							for (int i=revs.nb; i>change_rev; i--){
								rev2ref[i] = rev2ref[i-1];
							}
							rev2ref[change_rev] = 0;
						}
					}
				}else if (val == PAGE_VAL_USER3){
					int efface;
					bfusion_addrev (refs,one,change_ref,efface,1);
					if (change_ref >=0){
						// On a ajout ou retirer une colonne
						// Il faut donc synchroniser rev2rev
						int add = efface ? -1 : 1;
						for (int i=0; i<revs.nb; i++){
							if (rev2ref[i] >= change_ref){
								rev2ref[i] += add;
							}
						}
					}
				}
				if (change_ref >= 0 || change_rev >= 0){
					if (change_ref >= 0){
						page.closewin();
					}else{
						combine.closewin();
					}
					bfusion_setgrille(combine,revs,refs,rev2ref);
					if (change_ref >= 0){
						sp.linedesign();
						page.coldesign();
						page.draw();
					}else{
						combine.draw();
					}
				}
			}
		}
	}
	return ret;
}

