/*
	Gre la documentation associ  la livraison d'un projet.
	La documentation est inscrite dans un fichier change.dat.

	Ca permet  un programmeur de livrer en plusieurs session.
	Il modifie quelques fichiers et inscrit immdiatement les changements.
	Il ne termine pas la livraison. Le fichier change.dat est cr.

	Il modifie d'autres fichiers et ajoute  change.dat.

	Le format de change.dat est:

	#fichier
		texte
		texte
	#fichier
		...

	Lors de la livraison finale, le programmeur n'aura qu'a confirmer que
	change.dat est complet et l'opration d'archivage sera non interactive.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <misc.h>
#include "projet.h"

/*
	Association entre un fichier et un petit texte indiquant les changements
	subies par le fichier
*/
PUBLIC CHANGE_FILE::CHANGE_FILE(const char *_fichier, const char *_doc)
{
	strcpy (fichier,_fichier);
	doc = strdup_err (_doc);
}
PUBLIC CHANGE_FILE::~CHANGE_FILE()
{
	::free (doc);
}
/*
	Change la documentation associ  un fichier
*/
PUBLIC void CHANGE_FILE::setdoc(const char *_doc)
{
	::free (doc);
	doc = strdup_err (_doc);
}


/*
	Chargement du fichier change.dat.
	Ce n'est pas un erreur si le fichier n'existe pas.
*/
PUBLIC CHANGE_DAT::CHANGE_DAT(const char *_fname)
{
	fname = strdup_err (_fname);
	FILE *fin = fopen (fname,"r");
	if (fin != NULL){
		SSTRING buf,fichier;
		char line[1000];
		while (fgets(line,sizeof(line)-1,fin)!=NULL){
			strip_end (line);
			if (line[0] == '#'){
				if (!fichier.is_empty()){
					// On a collect la doc sur un fichier, on enregistre.
					list.add (new CHANGE_FILE (fichier.get(),buf.get()));
				}
				fichier.setfrom (line+1);
				buf.setfrom ("");
			}else{
				buf.appendf ("%s\n",line);
			}
		}
		if (!fichier.is_empty()){
			// On a collect la doc sur un fichier, on enregistre.
			list.add (new CHANGE_FILE (fichier.get(),buf.get()));
		}
	}
}

PUBLIC CHANGE_DAT::~CHANGE_DAT()
{
	free (fname);
}

static int cmp_file(const ARRAY_OBJ *p1, const ARRAY_OBJ *p2)
{
	CHANGE_FILE *f1 = (CHANGE_FILE *) p1;
	CHANGE_FILE *f2 = (CHANGE_FILE *) p2;
	return strcmp(f1->fichier,f2->fichier);
}

/*
	Tri la liste de fichier en ordre alphabtique.
*/
PUBLIC void CHANGE_DAT::sort()
{
	list.sort(cmp_file);
}
PUBLIC CHANGE_FILE* CHANGE_LIST::getitem(int no) const
{
	return (CHANGE_FILE*)ARRAY::getitem(no);
}
/*
	Trouve information associ  un fichier.
	Retourne NULL s'il n'y en a pas.
*/
PUBLIC CHANGE_FILE *CHANGE_DAT::locate (const char *fichier)
{
	CHANGE_FILE *ret = NULL;
	int nb = list.getnb();
	for (int i=0; i<nb; i++){
		CHANGE_FILE *file = list.getitem(i);
		if (strcmp(fichier,file->fichier)==0){
			ret = file;
			break;
		}
	}
	return ret;
}

/*
	Trouve la documentation associ  un fichier.
	Retourne NULL s'il n'y en a pas.
*/
PUBLIC const char *CHANGE_DAT::getdoc (const char *fichier)
{
	const char *ret = NULL;
	CHANGE_FILE *file = locate(fichier);
	if (file != NULL) ret = file->doc;
	return ret;
}

/*
	Enregistre la documentation associ  un fichier.
	Dtruit l'ancienne.
*/
PUBLIC void CHANGE_DAT::setdoc (const char *fichier, const char *doc)
{
	CHANGE_FILE *file = locate(fichier);
	if (file == NULL){
		list.add (new CHANGE_FILE (fichier,doc));
	}else{
		file->setdoc(doc);
	}
}
/*
	Sauve dans le fichier change.dat.
	L'information est toujours par ordre alphabtique de fichier.
	Retourne -1 si erreur.
*/
PUBLIC int CHANGE_DAT::save ()
{
	int ret = -1;
	FILE *fout = fopen_err (fname,"w",0);
	if (fout != NULL){
		sort();
		int nb = list.getnb();
		for (int i=0; i<nb; i++){
			CHANGE_FILE *file = list.getitem(i);
			fprintf (fout,"#%s\n\n%s\n",file->fichier,file->doc);
		}
		fclose (fout);
		ret = 0;
	}
	return ret;
}

