#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <misc.h>
#include "projet.h"
#include "projetx.m"

static char *log_path=NULL;
/* #Spcification: fichier log
	Plusieurs oprations tel l'archivage produisent un
	fichier enregistrant les diffrentes oprations qui ont t
	ralis.

	On retrouve ces fichiers dans le rpertoire o excute projetx.
	(n:\projet pour DOS). Gnralement ces fichiers ont l'extension
	".log".
*/
/* #Spcification: fichier log / emplacement
	Tous les fichiers log sont placs dans le mme rpertoire. Ce
	rpertoire est dfinie par la prfrence ADM/PROJET:logpath

	La valeur par dfaut de cette prfrence est
	$(applicationPath)/log

	Normalement le rpertoire log est dans le mme rpertoire que
	config, db et dialog.
*/
static PREFER_PATH logpath( "ADM/PROJET:logpath","/kit/log");

#
/*
	Enregistre le path o sont placer tous les fichiers log.
	Par dfaut, il sont placs l ou indique la prfrence ADM/PROJET:logpath.
*/
void log_setpath (const char *path)
{
	if (log_path != NULL) free (log_path);
	log_path = strdup_err (path);
}
/*
	Construit le path d'un fichier log  partir de son nom et du rpertoire
	prdfinie pour disposer les fichier log.
*/
void log_getpath (const char *fname, char *fpath)
{
	if (path_isabs (fname)){
		strcpy (fpath,fname);
	}else{
		if (log_path != NULL){
			path_make (log_path,fname,fpath);
		}else if (logpath.getval (fpath,MAXSIZ_PATH) != NULL){
			if (file_type (fpath) == -1
				&& file_mkdir (fpath) == -1){
				xconf_error (MSG_R(E_DIRCREATE),fpath);
				strcpy (fpath,fname);
			}else{
				path_make (fpath,fname,fpath);
			}
		}
	}
}
/*
	Ouvre un fichier texte dans le rpertoire des fichiers log.
	Ne retourne pas si ne peut pas ouvrir.

	L'appelant est responsable de fermer le fichier.
*/
FILE *log_fopen (const char *fname, int append)
{
	char fname_path[MAXSIZ_PATH];
	log_getpath (fname,fname_path);
	FILE *ret = fopen_err (fname_path,append ? "a" : "w" ,1);
	char datestr[11];
	char timestr[9];
	date_u2a (date_currentu(),datestr);
	time_u2a (time_currentu(),timestr);
	fprintf (ret,"---------- %s %s ----------\n",datestr,timestr);
	return ret;
}
/*
	Controle l'affichage des tapes de traitement excut par projet.
	Ce systme permet d'afficher ce que ralise projet au fur et  mesure.

	De plus, un fichier permet de retracer le traitement effectu
	durant une session.
*/
PUBLIC PROJET_LOG::PROJET_LOG(
	const char *fname,		// Enregistre dans le fichier fname si != NULL
	int append)				// Ajoute ou cration du log.
{
	fout = NULL;
	if (fname != NULL){
		fout = log_fopen (fname,append);
	}
}
PUBLIC VIRTUAL PROJET_LOG::~PROJET_LOG()
{
	if (fout != NULL) fclose (fout);
}
/*
	Enregistre une chaine dans le log.
	Cette fonction est redfinie pour accomoder les diffrents
	type de prsentation
*/
PROTECTED VIRTUAL void PROJET_LOG::sendstr(const char *str)
{
	if (fout != NULL){
		fprintf (fout,"%s",str);
	}
}

/*
	Ajoute un message dans le log.
	On peut utiliser cette fonction mme si le log n'est pas affich.
*/
PUBLIC void PROJET_LOG::vprintf (const char *ctl, va_list list)
{
	char buf[300];
	vsprintf (buf,ctl,list);
	sendstr (buf);
}


/*
	Ajoute un message dans le log.
	On peut utiliser cette fonction mme si le log n'est pas affich.
*/
PUBLIC void PROJET_LOG::printf (const char *ctl, ...)
{
	va_list list;
	va_start (list,ctl);
	vprintf (ctl,list);
	va_end   (list);
}

