#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <misc.h>
#include "projet.h"
#include "projetx.m"

/*
	Indique si le makefile.dat a t lue correctement.
	Mme si makefile.dat est vide ou existe pas, il contiendra toujours
	au moins 1 entre, soit lui-mme.
*/
PUBLIC int MAKEFILE::isok()
{
	return readok;
}
/* #Spcification: Archivage / makefile.dat / principal
	Une rvision principal nous permet de localiser un makefile.dat
	principal. Par exemple, la version principal 0.8 est dcrite
	par le fichier /kit/ombre/_0_8.s/makefile.dat.

	Ce makefile.dat nous permet de dterminer la combinaison
	exacte des rvisions de projets associs  cette rvision principale.

	En fait quand un programmeur dit: Je travaille sur la version 0.8,
	cela ne veut pas dire qu'il existe une version 0.8 de tous les
	projets et sous-projets. Les projets n'volue pas tous en mme temps.
	Quand on travaille sur la version 0.8, cela veut dire que la rvision
	du makefile.dat principal est 0.8. Ce fichier nous aiguille
	(de faon transparente) vers les bonnes rvisions des projets.
*/
/* #Spcification: Archivage / makefile.dat / format
	Le fichier makefile.dat dtermine le contenu d'un rpertoire pour
	une certaine rvision. C'est un fichier texte. Chaque ligne
	prsente un fichier ou sous-rpertoire.

	Il y a les information suivantes sur chaques lignes, spars par
	des espaces.

	nom_du_fichier ou sous-rpertoire:

		Simplement son nom, sans path. Gnralement les
		nom sont en minuscule, car c'est manipulable directement
		par DOS et UNIX.

		Si la ligne dcrit un rpertoire, son nom apparaitra
		comme suit "rpertoire/makefile.dat". Le / sert
		de marqueur pour reconnaitre que la ligne dcrit un
		sous rpertoire. La prsence du makefile.dat est aussi
		importante. On peut dire en fait que chaque ligne dcrit
		un fichier, ce fichier tant soit local ou dans un sous-rpertoire.

	date de rvision:

		La date de rvision en format jj/mm/aa.

	Heure de rvision:

		L'heure est en format hh:mm:ss.

	Location du fichier dans /kit/ombre:

		On retrouve ici le path absolue du fichier dans /kit/ombre. On
		remarquera que ce path dbute par $(KITOMBRE). Cela fait que
		les makefile.dat sont transportable, de DOS  UNIX, mais
		aussi d'un environnement  l'autre.

	Numro de rvision du fichier:

		Le numro de rvision est en format numrique avec un point
		sparant chaque nombre. L'information de rvision est aussi
		disponible dans la location. Il y a une certaine redondance.
		Plusieurs outils interprte le makefile.dat. Il tait
		souhaitable que ces outils soient le plus indpendant
		possible de l'algorythme qui dfinie la location dans
		/kit/ombre  partir du numro de rvision.
*/
/* #Spcification: Archivage / makefile.dat / path
	Les paths contenu dans le fichier makefile.dat sont toujours
	en format UNIX. Ca permet les comparaisons de path avec strcmp
	sans problme. Il sont aussi relatif  une base connue par dfinition.
*/
#

/*
	Le fichier makefile.dat est l'interface vers un rpertoire virtuelle.
	Il indique le contenu du rpertoire (les fichiers), leur date de
	rvision, leur numro de rvision, et un path pour trouver une
	copie du fichier.
*/
PUBLIC MAKEFILE::MAKEFILE (
	const char *_fname,		// Si NULL -> makefile.dat
							// Si fname reprsente un rpertoire,
							// Prend le fichier makefile.dat
	bool silence,			// Ne gnre pas de message si
							// _fname n'existe pas.
	const char *_projet,	// Nom du projet ou sous-projet
	const USERINFO *_user)	// Programmeur
{
	init (_fname,silence,_projet,_user);
}
/*
	Charge le fichier makefile.dat  partir de sa rvision.
	Il indique le contenu du rpertoire (les fichiers), leur date de
	rvision, leur numro de rvision, et un path pour trouver une
	copie du fichier.

	Le fichier est cherch dans le rpertoire /kit/ombre/projet/rvision.

	Si le fichier n'existe pas, essai de l'extraire de l'archive RCS.
	L'absence du fichier n'est pas un erreur.
*/
PUBLIC MAKEFILE::MAKEFILE (
	const char *_fname,		// Si NULL -> makefile.dat dans \kit\ombre
	bool silence,			// Ne gnre pas de message si
							// _fname n'existe pas.
	const REVISION *rev,
	const char *_projet,	// Nom du projet ou sous-projet
	const USERINFO *_user)	// Information sur environnement programmeur
{
	// Localise le path
	user = _user;
	char revpath[MAXSIZ_PATH];
	if (_fname == NULL) _fname = "makefile.dat";
	char ombpath[MAXSIZ_PATH];
	{
		_user->makombpath(_projet,ombpath);
		rev->formatpath (revpath);
		path_make (ombpath,revpath,ombpath);
		strcat (ombpath,".s");
		path_make (ombpath,_fname,revpath);
	}
	if (!file_exist(revpath)){
		char version_path[MAXSIZ_PATH];
		_user->makombpath(_projet,version_path);
		path_make (version_path,"version.dat",version_path);
		VERSION_DAT v(version_path);
		if (v.get(rev)!=NULL){
			// On est certain que cette rvision doit exister
			unzip (ombpath,rev,_fname);
		}
	}
	if (!file_exist(revpath)){
		/* #Spcification: makefile.dat / location possibles
			Lorsqu'on charge un makefile.dat d'un projet
			sans spcifier (on est pas interess  un en particulier,
			par exemple, celui dans le rpertoire de l'usager), on cherche
			dans /kit/ombre. Si on trouve pas (mme aprs le unzip)
			on cherche dans /kit/build.

			On retrouve cette situation lorsqu'on prpare une nouvelle
			fusion dans un version non encore livre dans /kit/build.

			Comme la version n'a pas t livre, il n'y en a pas trace
			dans /kit/ombre.
		*/
		char usrpath[MAXSIZ_PATH];
		_user->makusrpath(_projet,usrpath);
		path_make (usrpath,_fname,revpath);
	}
	init (revpath,silence,_projet,_user);
}


PUBLIC MAKEFILE::~MAKEFILE()
{
	delete [] tbfile;
}

/*
	Localise l'information dans le fichier makefile.dat
	sur un fichier.
	Retourne NULL si ne trouve pas.

	Un fichier qui a t limin via delfil() n'est plus
	accessible via locate().
*/
PUBLIC MAKEFILE_FILE *MAKEFILE::locate (
	const char *fname,
	bool finddeleted)		// Cherche aussi les fichiers effacs
{
	MAKEFILE_FILE *ret = NULL;
	char u_fname[MAXSIZ_PATH];
	path_2unix (fname,u_fname);
	MAKEFILE_FILE *file = tbfile;
	for (int i=0; i<nbfile; i++, file++){
		if ((finddeleted || !file->isdel())
			&& strcmp(u_fname,file->nom)==0){
			ret = file;
			break;
		}
	}
	return ret;
}

/*
	Localise l'information dans le fichier makefile.dat
	sur un fichier.
	Retourne NULL si ne trouve pas.

	Un fichier qui a t limin via delfil() n'est plus
	accessible via locate().
*/
PUBLIC MAKEFILE_FILE *MAKEFILE::locate (const char *fname)
{
	return locate (fname,false);
}

/*
	Localise un fichier et ajoute si absent.
	Retourne info sur fichier.
*/
PROTECTED MAKEFILE_FILE *MAKEFILE::locateadd (const char *fname)
{
	MAKEFILE_FILE *file = locate (fname);
	if (file == NULL){
		// Nouveau fichier ou fichier qui rapparait
		file = tbfile + nbfile++;
		strcpy (file->nom,fname);
		file->path = NULL;
		file->efface = 0;
	}
	return file;
}

/*
	Slectionne l'environment de l'usager comme path courant
*/
PROTECTED int MAKEFILE::setusrpath (SAVEPATH *save)
{
	char path[MAXSIZ_PATH];
	user->makusrpath(projet,path);
	return path_pushdir (user,path,save);
}
/*
	Extrait un fichier de l'archive compress et installation dans
	le bon rpertoire de /kit/ombre
*/
PROTECTED int MAKEFILE::unzip (
	const char *abs_ombpath,
	const REVISION *rev,
	const char *nom)
{
	/* #Spcification: archivage / stratgie de compaction
		Le contenu du rpertoire /kit/ombre augmente sans cesse.
		De temps en temps, une procdure (manuelle) est appliqu
		pour transferer tous les fichiers de kit/ombre dans un
		fichier pack.zip. La procedure est un script UNIX dans
		le fichier adm/projet/ziparch.

		L'organisation d'un rpertoire projet de /kit/ombre est donc:

		X/
		X/version.dat
		X/pack.zip
		X/sous-projet1
		X/...
		X/dernier-sous-projet
		X/_0_0.s
		X/...
		X/_1_2_3_4.s
		X/...

		Aprs la commande ziparch, les rpertoire *.s sont ajouts
		dans pack.zip et effac ensuite.

		Le rsultat est:

			-Rduction de la taille de l'archive d'environ 2/3
			-Rduction du nombre de fichier. Pour les systmes tel
			 DOS et Novell qui ont un systme d'allocation lmentaire,
			 chaque fichier occupe minimalement 4k (partition large).

		Si un fichier est absent de /kit/ombre, on tente de l'extraire
		du fichier pack.zip du rpertoire /kit/ombre correspondant.
		On utilise la commande unzip. Elle doit tre disponible
		dans le path.
	*/
	int ret = -1;
	if (file_type(abs_ombpath) == 1
		|| file_mkdiranc(abs_ombpath) != -1){
		char wpath[MAXSIZ_PATH];
		path_splitlex (abs_ombpath,wpath,NULL);
		/*
			Pour un projet X, si file reprsente un fichier, on a:
			abs_ombpath == /kit/ombre/X
			nom == nom sans aucun sous-rpertoire.

			Si file reprsente un sous-rpertoire:
			abs_ombpath == /kit/ombre/X/sous-rpertoire
			nom == sous-rpertoire/makefile.dat
		*/
		char revstr[MAXSIZ_PATH];
		rev->formatpath(revstr);
		strcat (revstr,".s");
		char fnom[MAXSIZ_NAME];
		path_splitlex (nom,NULL,fnom);
		ret = rcs_unzip (user,wpath,revstr,fnom);
	}
	return ret;
}
/*
	Extrait une rvision d'un fichier ou sous-rpertoire dans /kit/ombre

	Si force == 0, alors extraction seulement si le fichier est absent
	du rpertoire ombre.

	Retourne -1 si erreur. Signale l'erreur interactivement.
*/
PUBLIC int MAKEFILE::extract (MAKEFILE_FILE *file, int force)
{
	int ret = -1;
	char dest[MAXSIZ_PATH];
	user->makombpath (file->getrelpath(),dest);
	char abs_ombpath[MAXSIZ_PATH];
	path_splitlex (dest,abs_ombpath,NULL);
	ret = file_exist(dest) ? 0 : -1;
	if (force || ret == -1){
		ret = unzip(abs_ombpath,file->getrev(),file->nom);
	}
	if (ret == -1){
		/* #Spcification: archivage / extraction / dtection d'erreur
			Si un erreur est detecte durant l'extraction d'un fichier
			l'archive, on ajoute un message dcrivant le probleme dans
			le fichier error.log
		*/
		user->logerr ("Ne peut extraire %s\n",dest);
		char buf[1000];
		snprintf (buf,sizeof(buf)-1,MSG_U(Q_EXTRACT
			,"Ne peut localiser ou extraire correctement\n"
			 "\n%s\n\n"
			 "L'archive est probablement corrompue\n"
			 "Voulez vous continuer quand mme ?")
			,dest);
		if (xconf_yesno ("",buf,help_nil) == MENU_YES){
			/* #Spcification: archivage / extraction / dtection d'erreur
				Si un erreur est detecte durant l'extraction d'un fichier
				l'archive, on demande  l'usager si il veut passer l'erreur
				sous-silence. Ca permet de continuer une procdure et de
				corriger aprs le problme. Ca rend le programme plus tolrant.
		
				Evidemment cette stratgie n'est applicable que dans un
				cas o l'intgrit de l'archive n'est pas mise en danger.
			*/
			ret = 0;
		}
	}
	return ret;
}
/*
	Corrige le no de rvision d'un fichier.
	La fonction dterminera le numro de rvision requis.

	Tout les fichiers traits ici porteront le
	mme numro de rvision, soit le prochain numro de livraison
	du fichier makefile.dat principal. La version courante est
	contenu dans le champs user.

	Si le fichier ne fait pas partie de makefile.dat, il est ajout.
	Retourne NULL si erreur.

	ATTENTION: Le rpertoire courant doit tre celui du projet dans
			   l'environnement usager.
*/
PUBLIC MAKEFILE_FILE *MAKEFILE::setrev (
	const char *fname,
	REVISION_TYPE revtype,
	REVISION *oldrev)		// Contiendra une copie de l'ancien numro
							// de rvision.
{
	MAKEFILE_FILE *file = locateadd(fname);
	file->date = file_dosdate(fname);
	*oldrev = file->rev;
	file->pushrev (NULL);
	user->setrevnxt(&file->rev,revtype);
	file->setombpath(projet);
	is_modif = 1;
	return file;
}

/*
	Corrige la rvision d'un fichier de faon absolue.
*/
PUBLIC MAKEFILE_FILE *MAKEFILE::setabsrev (
	const char *fname,
	long date,			// Date du fichier
	const REVISION *absrev)	// Nouveau numro de rvision
{
	MAKEFILE_FILE *file = locateadd(fname);
	file->date = date;
	file->pushrev(absrev);
	file->setombpath(projet);
	is_modif = 1;
	return file;
}
/*
	Enregistre le nouveau numro de rvision d'un fichier.
*/
PUBLIC MAKEFILE_FILE *MAKEFILE::setrev (
	const char *fname,
	const REVISION *newrev,
	const char *path,	// Path ou le fichier a t dsarchiv.
						// Si c'est un path relatif, assume
						// qu'il est relatif  ombre.
	unsigned long date,	// Date de rvision du fichier.
	int archivable)		// Fichier archivable ou complmentaire.
{
	MAKEFILE_FILE *file = locateadd(fname);
	file->date = date;
	file->pushrev (newrev);
	if (file->path != NULL) free (file->path);
	file->archive = archivable;
	char relpath[MAXSIZ_PATH];
	if (path_abs2rel (path,user->getombre(),relpath) != 0){
		strcpy (relpath,path);
	}
	path_2unix (relpath,relpath);
	file->path = strdup_err (relpath);
	is_modif = 1;
	return file;
}

/*
	Retourne la structure associ au makefile.dat lui-mme.
	Ne peut pas retourner NULL.
*/
PUBLIC MAKEFILE_FILE *MAKEFILE::toi()
{
	assert (moi != NULL);
	return moi;
}

/*
	Efface tous les versions locales des fichiers qui ont t livrs.

	Retourne -1 si erreur.
*/
PUBLIC int MAKEFILE::cleanup ()
{
	int ret = -1;
	SAVEPATH save;
	if (setusrpath (&save) != -1){
		ret = 0;
		MAKEFILE_FILE *file = tbfile;
		for (int i=0; i<nbfile; i++,file++){
			if (file->efface){
				if (file_unlink (file->nom) == -1){
					xconf_error (MSG_U(E_CANTDELETE
						,"Ne peut effacer le fichier\n%s")
						,file->nom);
					ret = -1;
					break;
				}
			}
		}
		path_popdir (&save);
	}
	return ret;
}

/*
	Extrait tout les fichiers ou les rpertoire d'un projet vers ombre.
	Retourne -1 si erreur.
*/
PUBLIC int MAKEFILE::extract (int extr_file)
{
	int ret = 0;
	MAKEFILE_FILE *file = tbfile;
	for (int i=0; i<nbfile && ret == 0; i++,file++){
		if (file->is_dir){
			if (!extr_file){
				// Extrait les sous-rpertoires. Traits dans prjenv.c
				// plutot.
			}
		}else if (file->archive && file != moi){
			ret = extract (file,0);
		}
	}
	return ret;
}


/*
	Retourne le path relatif du fichier dans le systme de rfrence ombre.
	C'est le path vers la version originale.
	Si fichier == NULL, alors retourne le path original du makefile.dat.
*/
PUBLIC const char *MAKEFILE::getref (const char *fichier)
{
	const char *ret = NULL;
	MAKEFILE_FILE *file = fichier == NULL ? moi : locate(fichier);
	if (file != NULL){
		ret = file->getref();
	}
	return ret;
}

#ifdef TEST

void error_showdef (int noerr, ...)
{
	fprintf (stderr,"Error #%d\n",noerr);
	exit (-1);
}

int main (int argc, char *argv[])
{
	USERINFO user(prog_getid(true),"ref"
		,EVOL "/kit/groupe2",EVOL "/kit/groupe2"
		,EVOL "/kit/ombre");
	MAKEFILE mkf (NULL,0,"vdi",&user);
	mkf.ecrit ();
	return 0;
}

#endif

