#include <stdlib.h>
#include <string.h>
#include <misc.h>
#include "projet.h"
#include "projetx.m"

/*
	Enregistre la nouvelle version du makefile.dat dans version.dat
*/
PROTECTED int MAKEFILE::logversion (
	const char *nomver,			// Nom symbolique de la nouvelle rvision
	const REVISION *newrev,		
	const char *comment)		// Commentaires ajouts dans version.dat
{
	/* #Spcification: livraison / version.dat / principal
		Lors d'une livraison, un entre est ajout dans le version.dat
		de chaque projet livr. Cet entre identifie la rvision.
		Elle est suivie d'un commentaire identifiant la rvision de
		provenance.

		exemple:
			v42     	jacques 	0.0.2.2        	14/07/92
			;Livraison de 0.0.2.1
	*/
	VERSION_DAT ver(user,projet);
	VERSION_ONE *one = ver.add (nomver,user->getnom(),newrev);
	{
		char line[1000];
		char *pt = line;
		while (*comment != '\0'){
			if (*comment == '\n'){
				*pt = '\0';
				one->addinfo (line);
				pt = line;
				comment++;
			}else{
				*pt++ = *comment++;
			}
		}
		if (pt > line){
			*pt = '\0';
			one->addinfo (line);
		}
	}
	if (projet[0] == '\0'){
		/* #Spcification: livraison / version.dat / principal
			Pour le fichier version.dat principal, un commentaire est
			ajout pour chaque sous-projet qui ont t livrs durant
			la mme session

			Ces commentaires permettent de dterminer la provenance de la
			rvision.

			exemple:
				v42     	jacques 	0.0.2.2        	14/07/92
				;Livraison de 0.0.2.1
				; projet page
				; projet workbox
		*/
		// Patch etrangoide.
		// Recense tous les */makefile.dat de meme rvision que makefile.dat
		// courant, pour retrouver les projets qui ont t livrs
		// en mme temps.
		setiter();
		MAKEFILE_FILE *file;
		while ((file=iter("*/makefile.dat"))!=NULL){
			if (file->rev.cmp(&moi->rev)==0){
				char prj[MAXSIZ_NAME];
				path_splitlex (file->nom,prj,NULL);
				one->addinfo (" projet %s",prj);
			}
		}
	}
	{
		/* #Spcification: livraison / version.dat partiel
			Lors de la livraison d'un rpertoire, la composante ajoute
			au version.dat de ce rpertoire est copi dans un version.dat
			partiel de l'environnement /kit/livre.
		*/
		char revstr[MAXSIZ_REVISION+1];
		moi->rev.formatpath (revstr);
		char path[MAXSIZ_PATH];
		user->maklivpath(revstr,path);
		path_make (path,projet,path);
		path_make (path,"version.dat",path);
		FILE *fout = fopen_err (path,"w",0);
		if (fout != NULL){
			one->save (fout);
			fclose (fout);
		}
	}
	return ver.save(user);
}
/*
	Note que la version locales devra tre effac.
	Extrait la version dans le rpertoire ombre.

	ATTENTION: Le rpertoire courant doit tre celui du projet dans
			   l'environnement usager.

*/
PROTECTED int MAKEFILE::archive (MAKEFILE_FILE *file, const char *)
{
	int ret = -1;
	/* #Spcification: Archivage / RCS / historique de l'abandon
		Initialement, le programme projetX a t conu pour piloter
		RCS qui s'occupait d'archiver les diffrentes versions des
		fichiers sources.

		Comme les transactions avec l'archive (l'extraction d'une
		version par exemple) sont relativement lente, un rpertoire
		tampon a t cr. Il contient simplement toutes les variations
		de tous les sources. Les fichiers makefile.dat y font rfrence.
		C'est le rpertoire /kit/ombre.

		La vie a dmontrer que RCS tait plus ou moins utile.
		Voici les constatations qui nous amne en ce soir de septembre
		 mettre RCS sur la tablette:

		1-Il est bugg. La version DOS a simplement t compil et
		 peu prs rien n'a t fait pour contrecarrer les limites de DOS.
		Plus spcifiquement, les noms de fichier limits.

		2-Il est bugg. L'indentation de certain source n'est pas conserv.

		3-Il est lent.

		4-On ne peut pas insrer de rvision dans n'importe quel ordre.
		Pour cet aspect, la politique de RCS n'est pas compatible  la
		notre. Dans RCS, les numros de rvisions sont allous par
		fichier. Dans notre systme, ils sont impos par la rvision
		du projet. Par exemple, tous les sources rviss pour produire
		une nouvelle rvision d'un projet porteront ce numro de rvision.

		5-Et finalement, /kit/ombre est une duplication de l'archive
		et peut donc remplir la mme fonction. Et ce sans restriction
		sur l'insertion des rvisions.
	*/
	/* #Spcification: Archivage / RCS / Avenir de RCS
		RCS sera peut-tre utilis pour dcharg /kit/ombre des
		version moins utile. Son usage sera associ au mcanisme
		de mnage de /kit/ombre.

		... X annes plus tard ...

		Finalement, c'est le programme zip qui a remplac RCS pour
		faire le mnage dans /kit/ombre. C'est moins efficace que
		RCS, mais plus rapide et plus souple.
	*/
	/* #Spcification: Archivage / rpertoires / /kit/ombre
		Le rpertoire /kit/ombre contient un sous rpertoire pour tous
		les projets existant. Ces sous-rpertoires contiennent eux-mmes
		les sous-rpertoires reprsentant les sous-projets. etc ...

		Chacun des rpertoire contient des sous-rpertoires dont
		le nom reprsente un numro de rvision.

		exemple /kit/ombre/vdi/osdos/_0_1_2_1.s

		Ces rpertoires-versions contiennent des fichiers sources
		de la rvision correspondante.

		Attention: Une rvision X d'un projet contient des sources
		de mme rvision (X) et partage des sources d'autre rvision.

		exemple: La rvision 0.1.2.1 du projet vdi/osdos est compos
		de sources portant la rvision 0.1.2.1 ainsi que de fichiers
		sources provenant d'autre rvision. Il est probable que la
		rvision 0.1.2.1 est majoritairement compos de sources
		de rvision 0.1 et 0.0, et de quelques sources de rvision
		0.1.2.1.
	*/
#if 0
	char pathref[MAXSIZ_PATH];
	user->makrefpath(projet,pathref);
	if (rcs_put (file->nom,user->getnom(),&file->rev,pathref,doc) != -1){
		// On force l'extraction mme si le fichier est dj l.
		// En principe, c'est impossible que cette version  du fichier
		// puisqu'on vient tout juste de l'archiver. Ca dpanne pour
		// la phase de debug du systme, spcialement bootrcs.c. On
		// archive  rptition la mme chose.
		if (extract(file,1)!=-1){
			// Note la nouvelle rvision et note que la version
			// locale du fichier devra tre effacer.
			file->efface = 1;
			ret = 0;
		}
	}
#else
	/* #Spcification: Archivage / Oprations
		L'archivage se fait en deux oprations. On copie le fichier
		dans le rpertoire appropri de /kit/ombre et on le copie
		ensuite dans /kit/livre.
	*/
	if (foper_archive (file->nom,projet,&file->rev,user) != -1
		&& foper_backup (file->nom,projet,&file->rev,user) != -1){
		file->efface = 1;
		ret = 0;
	}
#endif
	return ret;
}
/*
	Livraison d'une rvision de fichier dans l'archive.
	La fonction dterminera la numro de rvision requis.
	Le fichier sera copi dans l'archive.
	Si tous va bien, le fichier fname sera dtruit.

	Tout les fichiers archivs avec cette instance porteront le
	mme numro de rvision, soit le prochain numro de livraison
	du fichier makefile.dat lui-mme.

	Si le fichier ne fait pas partie de makefile.dat, il est ajout.
	Retourne -1 si erreur.
*/
PUBLIC int MAKEFILE::livre (
	const char *fname,
	REVISION_TYPE revtype,
	const char *doc,		// Documentation expliquant les diffrences
							// avec la version prcdante.
	const SSTRINGS &tbnew,	// Fichiers jamais livr, mais accept.
	bool &doarch)			// Sera vrai si le fichier a vraiment t livr
							// (si le fichier n'a pas chang, il n'est pas
							//  livr, mais simplement effac)
{
	int ret = -1;
	SAVEPATH save;
	doarch = false;
	if (setusrpath (&save) != -1){
		MAKEFILE_FILE *file = locate (fname,true);
		doarch = true;
		if (file != NULL){
			char path[MAXSIZ_PATH];
			user->makombpath(projet,path);
			char revstr[MAXSIZ_PATH];
			file->rev.formatpath (revstr);
			strcat (revstr,".s");
			path_make (path,revstr,path);
			path_make (path,fname,path);
			file->is_del = 0;
			if (foper_compare (fname,path)==0){
				doarch = false;
				ret = 0;
				file->efface = 1;
				foper_logprintf (MSG_B(I_NOTCHANGED
					,"Fichier inchang, efface %s\n"
					,"File not changed, erase %s\n")
					,fname);
			}
		}else{
			// Nouveau fichier
			char path[MAXSIZ_PATH];
			path_make (projet,fname,path);
			if (tbnew.lookup(path)!=-1){
				doarch = true;
			}else{
				// Ok, nous n'archivons pas ce fichier, mais c'est ce que nous
				// voulions
				doarch = false;
				ret = 0;
			}
		}
		if (doarch){
			REVISION oldrev;
			MAKEFILE_FILE *file = setrev(fname,revtype,&oldrev);
			ret = archive (file,doc);
			if (ret == -1){
				file->rev = oldrev;
			}
		}
		path_popdir (&save);
	}
	return ret;
}

/*
	Livraison du fichier makefile.dat lui-mme.

	Retourne -1 si erreur.

	Aprs la livraison, on peut effacer (MAKEFILE::cleanup()) les copies
	locales des fichiers livrs.
*/
PUBLIC int MAKEFILE::livretoi (
	REVISION_TYPE revtype,
	bool keep,		// Garde une copie locale du makefile.dat
	const char *newversion,
	const char *comment)	// Commentaire
{
	int ret = -1;
	SAVEPATH save;
	/* #Spcification: livraison / makefile.dat
		Voici les oprations pour raliser la livraison du makefile.dat.

		Il doit tout d'abord s'crire sur disque et se livrer
		comme un fichier ordinaire. Il crit seulement la partie
		qui dcrit ce qui provient de /kit/ombre. Les fichiers
		de /kit/build qui se sont ajouts lors de la commande du projet
		sont exclus de l'archivage.

		Aprs la livraison, il se rcrit compltement sur disque
		(incluant les composantes de /kit/build).

		Le fichier version.dat enregistre la spcification de
		la nouvelle livraison.

		Il doit ensuite rviser le makefile.dat anctre (pour lui indiquer
		le changement de rvision).

		Optionnellement, il devra rester prsent dans le rpertoire usager.
		(L'opration de livraison ne doit pas interrompre le travail du
		programmeur. Aprs une livraison, son rpertoire doit tre logiquement
		dans le mme tat. Il peut continuer  travailler et compiler).

		On peut ensuite faire le cleanup si requis, soit effacer les
		versions locales des fichiers qui ont t livrs.
	*/
	if (setusrpath (&save) != -1){
		REVISION oldrev;
		MAKEFILE_FILE * file = setrev(moi->nom,revtype,&oldrev);
		// On force la date courante puisque le fichier va etre crit.
		file->date = ((unsigned long)date_currentu()<<16) + time_currentu();
		if (file != NULL){
			if (ecrit(1) != -1
				&& archive (file,NULL) != -1){
				char revstr[MAXSIZ_REVISION];
				oldrev.format (revstr);
				SSTRING tmp;
				tmp.setfromf ("Livraison de %s\n",revstr);
				tmp.append (comment);
				if (logversion (
						newversion == NULL || newversion[0] == '\0'
							? user->getversion() : newversion
						,&moi->rev
						,tmp.get()) != -1){
					cleanup();
					if (keep){
						ecrit(0);
						// Efface le makefile.del
						char pathdel[MAXSIZ_PATH];
						file_chgext (fname,pathdel,".del");
						file_unlink (pathdel);
					}
					if (projet[0] != '\0'){
						// Corrige le makefile.dat anctre
						char anc_projet[MAXSIZ_PATH];	// Path du projet
						char rel_projet[MAXSIZ_NAME];	// Nom relatif du projet
						path_splitlex (projet,anc_projet,rel_projet);
						MAKEFILE anc (NULL,true,anc_projet,user);
						// Fabrique un path projet/makefile.dat
						char path[MAXSIZ_PATH];
						path_make  (rel_projet,file->nom,path);
						path_2unix (path,path);
						anc.setrev (path,&file->rev,file->path,file->date,1);
						//anc.ecrit  (0);
						/* #Spcification: livraison / makefile.dat anctre
							Le makefile.dat anctre est corrig pour tenir
							compte de la nouvelle rvision du sous-rpertoire
							qui vient d'tre livr. En sauvant le makefile.dat
							anctre, on produit un makefile.old. Il agira
							comme flag indiquant qu'au moins un sous-rpertoire
							a t livr et qu'il faut donc livr aussi le
							makefile.dat anctre.

							Ce mcanisme est requis lorsqu'il y a panne
							pendant un squence de livraison. On peut
							gnralement reprendre l'opration de livraison
							et ainsi le makefile.dat anctre sera livr, mme
							si aucune composante du rpertoire anctre ou
							de ses sous-rpertoires n'ont t livres.
						*/
						anc.save();
					}
					ret = 0;
				}
			}else{
				file->rev = oldrev;
			}
		}
		path_popdir (&save);
	}
	return ret;
}

/*
	Cration d'une nouvelle version a partir de ce makefile.dat
	La version du fichier est corrig, et le makefile.dat est simplement
	archiv.

	Gnralement, s'applique surtout au makefile.dat de la racine.
*/
PUBLIC int MAKEFILE::creation(
	const char *nomver,		// Nom symbolique de la version  crer
	const REVISION *newrev)	// Numro de cette rvision

{
	int ret = -1;
	char tmppath[MAXSIZ_PATH];
	projet_tmppath.getval (tmppath,sizeof(tmppath));
	SAVEPATH save;
	// Pour archiver le makefile.dat, il faut l'crire
	// sur disque. Le systme d'archive opre  partir
	// du fichier dans le rpertoire courant.
	// On slectionne un rpertoire temporaire.
	if (path_pushdir (user,tmppath,&save)!=-1){
		/* #Spcification: admin / cration d'une version
			Voici les oprations requises pour associ un nouveau
			numro de rvision et un nouveau nom  une rvision
			existente.

			Le rsultat est deux rvisions qui pointent  la mme
			combinaison de fichier, mais qui vont voluer de faon
			indpendante par la suite.

			Voici les tapes requises pour crer une rvision 2.0
			 partir d'une rvision 1.9.


			mkdir /kit/ombre/_2_0.s
			cp    /kit/ombre/_1_9.s/makefile.dat /kit/ombre/_2_0.s

			Edition de /kit/ombre/_2_0.s/makefile.dat pour changer
			sa propre date de rvision.

			mkdir /kit/livre/_2_0
			cp /kit/ombre/_2_0.s/makefile.dat /kit/livre/_2_0

			Ajoute la rvision 2.0 et son nom dans
			/kit/ombre/version.dat

			Cration de /kit/livre/_2_0/version.dat qui contient
			uniquement l'information sur la rvision 2.0.
		*/

		long date = ((long)date_currentu()<<16) + time_currentu();
		REVISION oldrev = moi->rev;
		setabsrev ("makefile.dat",date,newrev);
		// La cration des rpertoire dans /kit/ombre et
		// /kit/livre est fait par mak.archive
		if (ecrit("makefile.dat",1) != -1
			&& archive (moi,"") != -1){
			// On doit corriger le version.dat
			// et crer un version.dat dans /kit/livre
			char revstr[MAXSIZ_REVISION];
			oldrev.format (revstr);
			SSTRING tmp;
			tmp.setfromf ("cration  partir de %s\n",revstr);
			ret = logversion (nomver,newrev,tmp.get());
		}
		path_popdir (&save);
	}
	return ret;
}
/*
	Copie un fichier dans l'espace usager.

	Extrait de l'archive et place dans rpertoire ombre si ce n'est pas
	dj fait.

	Retourne -1 si erreur.
*/
PUBLIC int MAKEFILE::copy2usr (MAKEFILE_FILE *file)
{
	int ret = -1;
	if (extract(file,0)!=-1){
		char usrpath[MAXSIZ_PATH];
		user->makusrpath (projet,usrpath);
		path_make (usrpath,file->getnom(),usrpath);
		char ombpath[MAXSIZ_PATH];
		user->makombpath (file->getrelpath(),ombpath);
		ret = file_copyp (ombpath,usrpath);
	}
	return ret;
}

/*
	Copie le makefile.dat dans l'espace usager.
	Retourne -1 si erreur.
*/
PUBLIC int MAKEFILE::copy2usr ()
{
	return copy2usr (moi);
}

