/* Spcification: programme / pdiff

	pdiff permet d'appliquer la commande diff entre deux fichiers
	quelquoncques ou deux versions d'une mme fichier. Il localise
	les fichiers soit localement ou dans l'archive (/kit/ombre) et
	excute la commande diff avec les arguments corrigs.

	Comparaison entre la version locale et le rfrence:

		pdiff [options de diff] fichier

	Comparaison entre deux fichiers:

		pdiff [options de diff] fichier1 fichier2
*/
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <misc.h>
#include "projet.h"

static void near usage()
{
	fprintf (stderr,
		"pdiff [ --diffopt ... ] fichier1 fichier2\n"
		"pdiff [ --diffopt ... ] fichier\n"
		"\n"
		"pdiff [ --diffopt ... ] [ --filecmd path ] [ --dir subdir ] --version revision1 revision2\n"
		);
}

/*
	Dtermine le path rel d'un fichier dans l'archive
*/
static void near pdiff_rpath (
	const char *fname,
	char *rpath,
	USERINFO *user)
{
	int ret = -1;
	MAKEFILE mkf ("makefile.dat",1,"",user);
	if (mkf.isok()){
		MAKEFILE_FILE *file = mkf.locate(fname);
		if (file != NULL){
			file->getombpath (user,rpath);
			ret = 0;
		}
	}
	if (ret == -1) exit (-1);
}

static void pdiff_log (const char *ctl, ...)
{
	va_list list;
	va_start (list,ctl);
	vfprintf (stderr,ctl,list);
	va_end (list);
}

int main (int argc, char *argv[])
{
	int ret = -1;
	etc_loadmsg();
	if (argc == 1){
		usage();
	}else{
		SSTRING cmd ("diff");
		// Recompose la ligne de commande
		// Copie les options.
		bool vmode = false;
		FILE *fcmd = NULL;
		SSTRINGS subdirs;
		int i;
		for (i=1; i<argc; i++){
			const char *arg = argv[i];
			if (strcmp(arg,"--diffopt")==0){
				cmd.append (" ");
				i++;
				cmd.append (argv[i]);
			}else if (strcmp(arg,"--version")==0){
				vmode = true;
			}else if (strcmp(arg,"--dir")==0){
				i++;
				subdirs.add (new SSTRING(argv[i]));
			}else if (strcmp(arg,"--filecmd")==0){
				i++;
				fcmd = fopen (argv[i],"w");
			}else{
				break;
			}
		}
		USERINFO *user = userinfo_findenv (prog_getid(true));
		if (user != NULL){
			if (vmode){
				if (argc != i + 2){
					usage();
				}else{
					ret = diff_findfiles (user,cmd.get(),argv[i],argv[i+1]
						,subdirs,stdout,fcmd,pdiff_log);
				}
			}else{
				// Il y a maintenant 1 ou deux noms de fichier.
				char *fname1 = argv[i];
				if (argc - i == 2){
					char *fname2 = argv[i+1];
				}else if (argc - i == 1){
					if (file_exist(fname1)){
						char rpath[MAXSIZ_PATH];
						pdiff_rpath (fname1,rpath,user);
						cmd.appendf (" %s %s",rpath,fname1);
						// printf ("# %s\n",cmd);
						ret = system (cmd.get());
					}else{
						fprintf (stderr,"Comparaison avec lui-mme\n");
					}
				}else{
					usage();
				}
			}
		}else{
			// Pas dans un rpertoire avec archive
			// appelle diff avec les arguments sans traitement
		}
	}
	return ret;
}

