#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <misc.h>
#include <dialog.h>
#include <etc.h>
#include "projet.h"
#include "prjlist.h"
#include "projetx.m"

/*
	Elimine les lments de tbsub[] de tb[].
	tb[] -= tbsub[].

	Retourne nombre d'lment restant dans tb[].
*/
static int tbstr_sub (
	char *tb[],
	int nbtb,
	char *tbsub[],
	int nbsub)
{
	for (int i=0; i<nbsub; i++){
		char *prj = tbsub[i];
		for (int j=0; j<nbtb; j++){
			if (tb[j] != NULL && strcmp(prj,tb[j])==0){
				free (tb[j]);
				tb[j] = NULL;
				break;
			}
		}
	}
	// Compacte la table.
	int nbret = 0;
	for (int i=0; i<nbtb; i++){
		if (tb[i] != NULL) tb[nbret++] = tb[i];
	}
	return nbret;
}
/*
	Construit une liste avec la slection des rpertoires projets.
*/
static int near projetp_getprjdir (
	const char *path,
	char *tbprj[],
	int maxtb)
{
	// /kit/ombre contient autant de repertoire que de projets
	// mais aussi autant de repertoire rvision que de projet
	// Dans DOS, le wild card "*" elimine les repertoires avec extensions.
	// Pas dans UNIX.
	// Donc on charge dans une plus grosse table qu'on copiera apres
	// avoir fait le menage.
	/* #Spcification: projets / kit/ombre / limite
		Le programme est limit a environ 100 projet dans /kit/ombre.
		De plus le nombre de sous-rpertoires incluant les projets et
		les rpertoires rvisions est limit  1000.
	*/
	char *tbtmp[1000];
	int nbprj = dir_getlistd (path,"*",0,tbtmp,1000);
	for (int i=0; i<nbprj; i++)	path_splitlex (tbtmp[i],NULL,tbtmp[i]);
	// Elimine les rpertoires qui ne sont pas des projets
	#if MISSING
		nbprj = tbstr_delfree (tbtmp,nbprj,"produit");
		nbprj = tbstr_delfree (tbtmp,nbprj,"doc");
		nbprj = tbstr_delfree (tbtmp,nbprj,"lib");
		nbprj = tbstr_delfree (tbtmp,nbprj,"h");
	#endif
	/* #Spcification: Archivage / nom de projet
		Le nom d'un projet ne doit pas commencer par un
		numro ni par un soulign '_'. C'est pour permettre
		de diffrencier les sous-rpertoires rvision des projets.

		On pourrait aussi faire la diffrence par l'extension,
		puisque les rpertoires rvision ont toujours l'extension ".s".

		La restriction sur la premire lettres aide  la prsentation
		des rpertoires lorsqu'on fait de l'entretien manuel dans
		/kit/ombre (en faisant un ls par exemple).
	*/
	for (int i=0; i<nbprj; i++){
		char *dir = tbtmp[i];
		char premier = dir[0];
		if (premier == '_' || isdigit(premier)){
			nbprj = tbstr_delfree (tbtmp,nbprj,dir);
			i--;
		}
	}
	if (nbprj > maxtb){
		xconf_error (MSG_U(E_PRJTOOMANY
			,"Trop de rpertoires projets dans %s: %d\n"
			 "    Maximum %d")
			,path,nbprj,maxtb);
		tbstr_free (tbtmp+maxtb,nbprj-maxtb);
		nbprj = maxtb;
	}
	memcpy (tbprj,tbtmp,nbprj*sizeof(tbprj[0]));
	return nbprj;
}
/*
	Construit une liste avec la slection des rvisions-projets
	Balaye un rpertoire et extrait les sous-rpertoires qui
	correspondent  des rvisions.

	Le rsultat est en ordre de rvision et non pas en ordre
	alphabtique. '_' > '9'.
*/
int projet_getrevdir (
	const char *path,
	char *tbrev[],
	int maxtb)
{
	int nbrev = dir_getlistd (path,"*",0,tbrev,maxtb);
	for (int i=0; i<nbrev; i++){
		char *rev = tbrev[i];
		path_splitlex (rev,NULL,rev);
		if (revision_path2rev(rev,NULL)==-1){
			nbrev = tbstr_delfree (tbrev,nbrev,rev);
			i--;
		}else{
			// Pour aider au tri, on remplace les _ par des -
			// On corrigera apres.
			while (*rev != '\0'){
				if (*rev == '_') *rev = '-';
				rev++;
			}
		}
	}
	tbstr_sort (tbrev,nbrev);
	for (int i=0; i<nbrev; i++){
		char *rev = tbrev[i];
		while (*rev != '\0'){
			if (*rev == '-') *rev = '_';
			rev++;
		}
	}
	return nbrev;
}
PUBLIC VIRTUAL PRJLISTE::~PRJLISTE()
{
}

PUBLIC VIRTUAL void PRJLISTE::setupdia (DIALOG &)
{
}

PROTECTED int PRJLISTE_GET::subuser (USERINFO *user, char *tbprj[], int nbprj)
{
	char *tbuser[100];
	int nbuser = dir_getlistd (user->getusrpath(),"*",0,tbuser,100);
	for (int i=0; i<nbuser; i++) path_splitlex (tbuser[i],NULL,tbuser[i]);
	// On limine les projets que l'usager a dj
	nbprj = tbstr_sub (tbprj,nbprj,tbuser,nbuser);
	tbstr_free (tbuser,nbuser);
	return nbprj;
}

/*
	Obtient la liste des projets pour un rvision.
*/
static int near prjlist_getrevdir (USERINFO *user, char *tbprj[], int maxtb)
{
	const REVISION *revsrc = user->getrevsrc();
	MAKEFILE master ((char*)NULL,false,revsrc,"",user);
	master.setiter();
	MAKEFILE_FILE *file;
	int nbprj = 0;
	while ((file=master.iter("*/makefile.dat"))!=NULL){
		char newprojet[MAXSIZ_PATH];
		path_splitlex (file->getnom(),newprojet,NULL);
		tbprj[nbprj++] = strdup_err(newprojet);
		if (nbprj == maxtb) break;
	}
	tbstr_sort (tbprj,nbprj);
	return nbprj;
}
static void near prjlist_copy2info (
	char *tbprj[],
	PRJ_INFO tbiprj[],
	int nbprj)
{
	for (int i=0; i<nbprj; i++) tbiprj[i].nom = tbprj[i];
}
/*
	Obtient la liste des projets pour un rvision moins ceux que
	l'usager a dj.

	Retourne le nombre de projet plac dans tbprj[] (strdup_err()).
*/
PUBLIC int PRJLISTE_GET::getlist(
	USERINFO *user,
	PRJ_INFO tbiprj[],
	int maxtb)
{
	char *tbprj[200];
	int nbprj = prjlist_getrevdir(user,tbprj,maxtb);
	nbprj = subuser (user,tbprj,nbprj);
	prjlist_copy2info (tbprj,tbiprj,nbprj);
	return nbprj;
}

/*
	Obtient la liste des tous les projets moins ceux que l'usager a dj
	et moins ceux qui font partie de la rvision.

	Retourne le nombre de projet plac dans tbprj[] (strdup_err()).
*/
PUBLIC int PRJLISTE_GETALL::getlist(
	USERINFO *user,
	PRJ_INFO tbiprj[],
	int maxtb)
{
	char *tbprj[200];
	int nbprj = projetp_getprjdir (user->getombre(),tbprj,maxtb);
	char *tbrev[200];
	int nbrev = prjlist_getrevdir (user,tbrev,200);
	nbprj = tbstr_sub (tbprj,nbprj,tbrev,nbrev);
	tbstr_free (tbrev,nbrev);
	nbprj = subuser (user,tbprj,nbprj);
	prjlist_copy2info (tbprj,tbiprj,nbprj);
	return nbprj;
}

PUBLIC int PRJLISTE_GET::exec (
	USERINFO *user,
	int ,					// Opration command.
	const char *tbprj[],	// Projets slectionns
	int nbprj)
{
	// Le programmeur veut une copie du ou des projets
	return projet_setup (user,tbprj,nbprj);
}

PUBLIC const char **PRJLISTE_GET::getbutton()
{
	static const char *tb[]={MSG_U(B_GET,"Prend"),NULL};
	return tb;
}

PUBLIC int PRJLISTE_LIVRE::getlist(
	USERINFO *user,
	PRJ_INFO tbiprj[],
	int maxtb)
{
	char *tbprj[200];
	int nbprj = projetp_getprjdir (user->getusrpath(),tbprj,maxtb);
	prjlist_copy2info (tbprj,tbiprj,nbprj);
	return nbprj;
}

PUBLIC void PRJLISTE_LIVRE::setupdia (DIALOG &dia)
{
	dia.newf_str (MSG_U(F_NEWVERNAME,"Nom de version(opt)"),newversion);
	dia.newf_textarea (MSG_U(F_COMMENT,"Commentaire"),comment,40,5);
}


PUBLIC int PRJLISTE_LIVRE::exec (
	USERINFO *user,
	int opr,				// Opration command.
	const char *tbprj[],	// Projets slectionns
	int nbprj)
{
	int ret = -1;
	if (nbprj == 0){
		xconf_error (MSG_R(E_AUMOINS1));
	}else{
		switch (opr){
		case 0:
			// Le programmeur ne veut plus de sa copie du projet
			{
			for (int i=0; i<nbprj; i++){
				PRJLOCK prjlock (user,tbprj[i]);
				if (prjlock.isok()){
					prjlock.del  (tbprj[i]);
					prjlock.save ();
					// Le programmeur veut faire du menage
				}		
			}
			ret = projet_deluserdir (user,tbprj,nbprj);
			}
			break;
		case 1:
			if (nbprj > 1){
				xconf_error (MSG_U(E_UNPRJ,"Un projet  la fois"));
			}else{
				// Documente les changements avant livraison
				change_edit (user,tbprj[0]);
				// Le -1 fait que le dialogue qui a dclench
				// cette fonction ne termine pas.
				ret = -1;
			}
			break;
		case 2:
			// Livre une rvision d'un projet
			ret = projet_livre (user,tbprj,nbprj,newversion.get(),comment.get());
			break;
		}
	}
	return ret;
}

PUBLIC const char **PRJLISTE_LIVRE::getbutton()
{
	static const char *tb[]={
		MSG_U(B_UNGET,"Abandon"),
		MSG_U(B_DOCUMENT,"Document"),
		MSG_U(B_LIVRE,"Livre"),
		NULL
	};
	return tb;
}
/*
	Extrait la liste de tous les projets qui font partie de la rvision cible.

	Retourne le nombre de projet dans tbprj[].
*/

PUBLIC int PRJLISTE_BUILD::getlist(
	USERINFO *user,
	PRJ_INFO tbiprj[],
	int maxtb)
{
	char *tbprj[200];
	int nbprj = prjlist_getrevdir (user,tbprj,maxtb);
	prjlist_copy2info (tbprj,tbiprj,nbprj);
	return nbprj;
}
/*
	Extrait la liste de tous les projets excluant ceux qui font
	partie de la rvision cible.

	Retourne le nombre de projet dans tbprj[].
*/
PUBLIC int PRJLISTE_BUILDALL::getlist(
	USERINFO *user,
	PRJ_INFO tbiprj[],
	int maxtb)
{
	char *tbprj[200];
	int nbprj = projetp_getprjdir (user->getombre(),tbprj,maxtb);
	prjlist_copy2info (tbprj,tbiprj,nbprj);
	return nbprj;
}
PUBLIC int PRJLISTE_BUILD::exec (
	USERINFO *user,
	int opr,			// Opration command.
	const char *tbprj[],	// Projets slectionns
	int nbprj)
{
	int ret = -1;
	if (nbprj == 0){
		xconf_error (MSG_R(E_AUMOINS1));
	}else{
		switch (opr){
		case 0:
			ret = projet_setupbuild (user,tbprj,nbprj);
			break;
		}
	}
	return ret;
}

PUBLIC const char **PRJLISTE_BUILD::getbutton()
{
	static const char *tb[]={MSG_U(B_BUILD,"Build"),NULL};
	return tb;
}
PUBLIC int PRJLISTE_LIVREB::getlist(
	USERINFO *user,
	PRJ_INFO tbiprj[],
	int maxtb)
{
	char path[MAXSIZ_PATH];
	user->makbldpath("",path);
	char *tbprj[200];
	int nbprj = projetp_getprjdir (path,tbprj,maxtb);
	prjlist_copy2info (tbprj,tbiprj,nbprj);
	return nbprj;
}

