/*
	controle interactif des transfert de fichier entre l'espace
	de l'usager et la rfrence de groupe
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <assert.h>
#include <misc.h>
#include <dialog.h>
#include "projet.h"
#include "projetx.m"

/*
	initialise le projet dans l'espace de l'usager.

	Cration des sous-rpertoires et extractions des fichiers makefile.dat.
	retourne -1 si abandon ou erreur.
*/
int projet_get (
	USERINFO *user,
	const char *tbprj[],
	int   nbprj,
	bool dolog,			// Enregistre que le projet a t pris
	bool getsub,		// Prend rcursivement les sous-rpertoires.
	bool getobj,		// Cration des sous-rpertoires pour compilation
	PROJET_LOG *log)	// Peut tre NULL
{
	int ret = -1;
	foper_setlog (log);
	char dpath[MAXSIZ_PATH];
	user->makusrpath("",dpath);
	if (file_mkdiranc (dpath) == -1){
		xconf_error (MSG_U(E_NOMKDIR
			,"Ne peut crer le rpertoire\n%s")
			,dpath);
	}else{
		// Localise et charge le makefile.dat principal
		const REVISION *revsrc = user->getrevsrc();
		if (log != NULL){
			char srcstr[MAXSIZ_REVISION];
			revsrc->format (srcstr);
			char dststr[MAXSIZ_REVISION];
			user->getrevdst()->format (dststr);
			log->printf ("Extraction de %s -> %s\n",srcstr,dststr);
		}
		MAKEFILE master ((char*)NULL,false,revsrc,"",user);
		if (!master.isok()){
			user->logerr ("Master pas ok\n");
		}else{
			ret = 0;
			for (int i=0; i<nbprj && ret == 0; i++){
				const char *projet = tbprj[i];
				if (log != NULL) log->printf ("%s/\n",projet);
				// Obtient le path du makefile.dat appropri pour
				// le projet.
				char makpath[MAXSIZ_PATH];
				path_make (projet,"makefile.dat",makpath);
				MAKEFILE_FILE *file = master.locate (makpath);
				if (file == NULL){
					/* #Spcification: installe / projet inexistant
						Lorsqu'on commande l'installation d'un projet
						qui n'existe pas dans la rvision que l'on
						a slectionn, il y a simplement cration
						d'un rpertoire vide.

						En fait, une nouvelle option permettra
						d'installer une rvision spcifique d'un
						projet dans une autre rvision.

						Par exemple, on slectionnera un rvision
						0.2 qui ne contient pas le projet X. On slectionne
						les projets de 0.2 sur lesquels on veut travailler.
						Ensuite on slectionne une rvision particulire
						du projet X que l'on installe dans l'environnement
						0.2.

						On peut dj simuler manuellement ce traitement.
						On installe la version 0.2 et ensuite la rvision
						qui contient le projet X. On copie ensuite le
						rpertoire X dans l'environnement 0.2.
					*/
					char dirpath[MAXSIZ_PATH];
					user->makusrpath(projet,dirpath);
					file_mkdir(dirpath);
					if (log != NULL){
						log->printf ("    Nouveau projet: %s\n",dirpath);
					}
				}else{
					master.extract (file,0);
					ret = prjenv_get (user,projet,file->getrelpath()
						,getsub,getobj,log);
					if (ret != -1){
						if (dolog){
							PRJLOCK prjlock (user,projet);
							if (prjlock.isok()){
								char path[MAXSIZ_PATH];
								user->makusrpath (projet,path);
								prjlock.add (0,projet,path);
								prjlock.save();
							}
						}
					}
				}
			}
		}
	}
	foper_setlog (NULL);
	return ret;
}
/*
	initialise le projet dans l'espace de l'usager.

	Cration des sous-rpertoires et extractions des fichiers makefile.dat.

	Affiche les oprations dans une fentre.
	retourne -1 si abandon ou erreur.
*/
int projet_get (
	USERINFO *user,
	const char *tbprj[],
	int   nbprj,
	int   dolog,		// Enregistre que le projet a t pris
	int   getsub,		// Prend rcursivement les sous-rpertoires.
	int	  getobj)		// Cration des sous-rpertoires pour compilation
{
	WINDOW_LOG log("prend",0);
	log.show();
	return projet_get (user,tbprj,nbprj,dolog,getsub,getobj,&log);
}
/*
	initialise le projet dans l'espace de l'usager.

	Cration des sous-rpertoires et extractions des fichiers makefile.dat.
	Retourne -1 si abandon ou erreur.
*/
int projet_setup (
	USERINFO *user,
	const char *tbprj[],
	int   nbprj)
{
	int ret = -1;
	if (projet_chkusrenv(user)!=-1){
		if (!user->nobuild()){
			// Vrifie que la version intgre est disponible dans /kit/build
			int nbok = 0;
			for (int i=0; i<nbprj; i++){
				const char *projet = tbprj[i];
				char path[MAXSIZ_PATH];
				user->makbldpath(projet,path);
				if (file_type(path) == 1){
					nbok++;
				}else{
					xconf_error (MSG_U(E_NOBUILD
						,"Pas de rpertoire d'intgration\n"
						 "pour le projet %s")
						,path);
				}
			}
			if (nbok == nbprj){
				ret = projet_get (user,tbprj,nbprj,true,true,false);
			}
		}else{
			ret = projet_get (user,tbprj,nbprj,true,true,false);
		}
	}
	return ret;
}

/*
	Vrifie que l'environnement usager existe et on commande
	la cration si requis.

	Retourne -1 si l'environnement n'est pas ok et que usager a refus
	de cr.	
*/
int projet_chkusrenv (USERINFO *user)
{
	int ret = 0;
	char usrpath[MAXSIZ_PATH];
	user->makusrpath ("",usrpath);
	char objpath[MAXSIZ_PATH];
	user->makobjpath ("",objpath);
	if (file_type(usrpath) != 1){
		ret = -1;
		char buf[PATH_MAX];
		sprintf (buf,MSG_U(Q_SETBUILDDIR,"Cration du rpertoire\n\n%s")
			,usrpath);
		if (xconf_yesno("",buf,help_nil) == MENU_YES){
			/*
				Cration de /kit/build/version.

				Les rpertoires doc et h sont crs dans prjenv_get()
				appel par projet_get().
			*/
			ret = file_mkdiranc (usrpath);
			if (!ret){
				// Il faut extraire le projet racine
				static const char *tb[]={""};
				ret = projet_get   (user,tb,1,false,false,false,NULL);
			}else{
				xconf_error (MSG_R(E_DIRCREATE),usrpath);
				user->logerr ("Ne peut pas creer le rpertoire %s (%d)\n"
					,usrpath,ret);
			}
		}
	}
	return ret;
}
/*
	Efface le contenu de l'environnement usager.
	Retourne 0 si ok, -1 si abandon ou erreur
*/

int projet_deluserdir (
	USERINFO *user,
	const char *tbprj[],
	int nbprj)
{
	int ret = -1;
	DIALOG dia;
	char srcdir = 1;
	char objdir = 1;
	dia.newf_chk ("",srcdir,MSG_U(F_DELUSR,"Efface les sources des projets"));
	dia.newf_chk ("",objdir,MSG_U(F_DELOBJ,"Efface les objets des projets"));
	//page.setspec (but);
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.edit (MSG_U(T_ABANDON,"Abandon de la rvision")
			,MSG_U(I_ABANDON
				,"Vous ne voulez plus travailler sur cette rvision du projet\n"
				 "et vous vous prparez  effacer les objets et les sources\n"
				 "de votre rpertoire personnel.")
			,help_nil,nof);
		if (code == MENU_CANCEL || code == MENU_ESCAPE){
			break;
		}else if (code == MENU_ACCEPT){
			for (int i=0; i<nbprj; i++){
				ret = prjenv_deldir (user,tbprj[i],srcdir,objdir);
				if (ret == -1) break;
			}
			break;
		}
	}
	return ret;
}

#if 0

void projet_deldirs (
	const char *prog,
	const char *projet)
{
	char objpath[MAXSIZ_PATH];
	char usrpath[MAXSIZ_PATH];
	projet_makusrpath (prog,projet,usrpath);
	projet_deldir (usrpath);
	if (projet_makobjpath(projet,objpath) == 1){
		projet_deldir (objpath);		
	}
}
#endif

static int projet_selectnew (SSTRINGS &tbnew)
{
	int ret = 0;
	if (tbnew.getnb()>0){
		ret = -1;
		DIALOG dia;
		int nb = tbnew.getnb();	
		char sel[nb];
		memset (sel,0,nb);
		for (int i=0; i<nb; i++){
			dia.newf_chk ("",sel[i],tbnew.getitem(i)->get());
		}
		dia.setbutinfo (MENU_USR1,MSG_R(B_ALL),MSG_R(B_ALL));
		dia.setbutinfo (MENU_USR2,MSG_R(B_NONE),MSG_R(B_NONE));
		int nof = 0;
		while (1){
			MENU_STATUS code = dia.edit (MSG_U(T_NEWFILES,"Nouveaux fichiers")
				,MSG_U(I_NEWFILE,"Voici la liste des nouveau fichier sources\n"
					" archiver. Slectionner ceux que vous souhaitez ajouter\n"
					" l'archive (les autres resteront l).")
				,help_nil,nof,MENUBUT_ACCEPT|MENUBUT_CANCEL|MENUBUT_USR1
					|MENUBUT_USR2);
			if (code == MENU_CANCEL || code == MENU_ESCAPE){
				break;
			}else if (code == MENU_USR1 || code == MENU_USR2){
				memset (sel,code ==MENU_USR1 ? 1 : 0 , nb);
				dia.reload();
			}else{
				// On doit conserver uniquement les nouveau fichiers
				// accept.
				int pos = 0;
				for (int i=0; i<nb; i++){
					if (!sel[i]){
						tbnew.remove_del(pos);
					}else{
						pos++;
					}
				}
				ret = 0;
				break;
			}
		}
	}
	return ret;
}

/*
	Archive les rvisions de l'usager et efface les fichiers locaux.
	Retourne -1 si erreur.
*/
int projet_livre (
	USERINFO *user,
	const char *tbprj[],
	int nbprj,
	const char *version,
	const char *comment)
{
	int ret = -1;
	PRJLOCK racine(user,"_racine");
	if (racine.isok()){
		// Essai d'obtenir un lock sur tout les projets
		PRJLOCK *tblock[nbprj];
		int nblock = 0;
		for (int i=0; i<nbprj; i++){
			tblock[nblock++] = new PRJLOCK (user,tbprj[i]);
			if (tblock[i] == NULL){
				nblock--;
				break;
			}else if (!tblock[i]->isok()){
				break;
			}
		}
		if (nblock == nbprj){
			WINDOW_LOG log ("livre.log",1);
			foper_setlog (&log);
			int err = 0;
			int nb = 0;
			PRJCTRL ctrl(user);
			SSTRINGS tbnew;
			for (int i=0; i<nbprj; i++){
				SSTRINGS tberr;
				int nberr = livre_test (user,&ctrl,tbprj[i],false,tberr,tbnew);
				if (nberr > 0){
					err = -1;
					for (int j=0; j<nberr; j++){
						log.printf ("%s\n",tberr.getitem(j)->get());
					}
				}
			}
			if (err == -1){
				xconf_error (MSG_U(E_LIVRETEST
					,"Certain fichier ne sont pas\n"
					 "livrables, soit parce que non documents\n"));
			}else if (projet_selectnew (tbnew) != -1){
				#if 0
					for (int i=0; i<tbnew.getnb(); i++){
						printf ("tbnew[%d] :%s:\n",i,tbnew.getitem(i)->get());
					}
				#endif
				log.show();
				REVISION_TYPE revtype = user->isbuild()
					? REVISION_INTEGRE : REVISION_LIVRE;
				for (int i=0; i<nbprj && err != -1; i++){
					err = livre_revision (user,&ctrl,&log,tbprj[i],""
						,revtype,version,tbnew);
					if (err != -1) nb += err;
				}
				// Livraison de la racine
				if (nb > 0
					&& livre_revision (user,&ctrl,&log,"",comment,revtype
						,version,tbnew)	!= -1){
					/* #Specification: livraison / succs
						Lorsque la livraison est un succs.
						Les makefile.dat ont t modifis. L'environnement
						de l'usager correspond donc  une nouvelle version.
						Il faut donc renommer les rpertoires pour rflter
						ce changement.

						Le rpertoire source est renomm ainsi que le
						rpertoire objet. Un erreur est signal si
						le rpertoire objets n'existe pas, mais
						ca ne nuit pas  l'intgrit de l'archive.
					*/
					user->chgenv(revtype,1,&log);
				}
			}
			foper_setlog (NULL);
		}
		for (int i=0; i<nblock; i++) delete tblock[i];
		ret = 0;
	}
	/* #Specification: Livraison / log
		Une trace des actions apparait dans un window.
		Un jour, ce window aura des scroll bars et l'usager
		pourra visionner l'ensemble du traitement en
		remontant.

		De plus, une copie est ajoute dans le fichier livre.log.
	*/
	return ret;
}

