
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <misc.h>
#include <dialog.h>
#include <tool.h>
#include "projet.h"
#include "prjlist.h"
#include "projetx.m"

/*
	Vrifie l'environnement d'excution et gnre des messages d'erreur
	Retourne -1 si environnement pas acceptable.
*/
static int near projet_chkenv (const char *prog)
{
	int ret = -1;
	char path[MAXSIZ_PATH];
	if (path_validate()==-1){
		xconf_error (MSG_U(E_IVLDPATH,"Variable PATH invalide"));
	}else if (prog_getnum(prog) == -1){
		xconf_error (MSG_U(E_UNKNOWNUSER
			,"L'usager %s est inconnue\n"
			 "Un numro d'identification\n"
			 "doit tre allou  cet usager")
			,prog);
	}else if (file_findexe("unzip",path)==-1){
		xconf_error (MSG_U(E_NOUNZIP
			,"La commande unzip n'est pas disponible.\n"
			 "Cette commande est essentielle  l'opration\n"
			 "de l'archive."));
	}else{
		const char *preload = getenv ("LD_PRELOAD");
		if (preload != NULL && strstr(preload,"projetx-vfs")!=NULL){
			xconf_error (MSG_B(E_NOVFS
				,"projetX ne peut excuter correctement en mode virtuel\n"
				,"projetX can't execute properly in virtual mode\n"));
		}else{
			ret = 0;
		}
	}
	return ret;
}


/*
	S'assure que la version est slectionne (ou force une slection).
	Retourne != 0 si une version est slectionne.
*/
static int near projetp_chkver (
	USERINFO *user,
	bool force)
{
	if (user->getversion()==NULL || force){
		version_setmain (user,NULL);
	}
	return user->getversion() != NULL;
}
static USERINFO *user;

static PREFER_INT minlivreespace ("minlivreespace",10000);
/*
	Vrifie s'il y a suffisamment d'espace pour livrer
	Retourne 1 si espace suffisant ou si l'usager livre quand mme.
*/
static int projet_chkspace(USERINFO *user)
{
	/* #Spcification: livraison / espace requis
		Avant chaque livraison, l'espace disque dans /kit/ombre et
		/kit/livre (test sparment mme si c'est dans le mme volume)
		est valu. S'il y a moins de 10 megs, alors l'usager doit
		confirm manuellement qu'il dsire vraiment livr quand mme.

		Idalement, projetx devrait valuer l'espace requis. Par contre
		il y a telle de chose qui peut arriver dans le mme volume
		au mme moment, qu'il est prfrable de jouer prudemment.

		Le 10 megs est en fait un prfrence (en K). La prfrence
		est
	*/
	long minesp = minlivreespace.getval();
	const char *kitlivre = user->getlivre();
	long spclivre = file_getfree (kitlivre);
	const char *kitombre = user->getombre();
	long spcombre = file_getfree (kitombre);
	int ret = 1;
	if (spcombre == -1 || spclivre == -1){
		char buf[1000];
		sprintf (buf,MSG_U(Q_CANTFREESPACE
			,"Ne peut dterminer l'espace libre\n"
			 "dans %s et %s\n"
			 "Voulez vous livrer quand mme ?")
			,kitombre,kitlivre);
		ret = xconf_yesno ("",buf,help_nil)==MENU_YES;
	}else if (spcombre < minesp || spclivre < minesp){
		char buf[1000];
		sprintf (buf,MSG_U(Q_NOFREESPACE
			,"Espace disponible insuffisant\n"
			 "%s : %ld K\n"
			 "%s : %ld K\n"
			 "Espace minimum suggr : %ld K\n"
			 "Voulez vous livrer quand mme ?")
			,kitombre,spcombre
			,kitlivre,spclivre
			,minesp);
		ret = xconf_yesno ("",buf,help_nil) == MENU_YES;
	}
	return ret;
}

static const char *u_version;
static const char *u_commande;
static const char *u_livraison;
static const char *u_menage;
static const char *u_fusion_simple;
static const char *u_fusion_complexe;
static const char *u_fusion_edit;
static const char *u_commande_autre;

static const char *c_compilation;
static const char *c_livraison;
static const char *c_fusion_prepare;
static const char *c_fusion_edit;
static const char *c_commande_autre;
static const char *c_diff;

static const char *a_initarch;
static const char *a_creation;
static const char *a_destruction;
static const char *a_menage;
static const char *a_importation;




/*
	Fonction qui active les principaux dialogues dpendant de la slection
	du menu.
*/
static void projet_main (const char *key)
{
	bool pasfait = false;
	if (key == u_livraison){
		/* #Spcification: menu / usager / Livraison
			Slectionne une rvision dans l'espace usager
			et livre les sources modifis dans l'archive.

			L'usager peut livrer slectivement les projets
			qu'il veut.

			La livraison introduit une nouvelle rvision
			dans l'archive correspondant  la nouvelle
			combinaison de sources des diffrents projets.

			Le rpertoire racine sera renomm pour
			confirmer l'association de l'environnement usager
			avec cette nouvelle rvision.

			L'opration de livraison est non destructive. On
			peut livrer et continuer  travailler dans la
			nouvelle rvision sans intervention particulire.

			Mme si des fichiers sont effacs, ils continuent
			d'tre "logiquement" disponible.
		*/
		char path[MAXSIZ_PATH];
		user->gethome(path);
		if (projet_chkspace(user)==1
			&& version_setmain (user,path)){
			PRJLISTE_LIVRE g;
			projet_select (user,g);
		}
	}else if (key == u_menage){
		// Fait le mnage de l'environnement usager
	}else if (key == u_fusion_simple){
		/* #Spcification: menu / usager / fusion / prepare simple
			La prparation du fusion dans l'espace usager consiste
			en la slection de deux rvisions pralablement install
			dans l'espace usager.

			La premire est la rvision temporaire qui contient
			des sources qui n'ont pas t livrs dans l'archive.

			La seconde est la rvision destination qui a t
			install sans aucune modification.

			Aprs la slection, projetx valuera les changements
			qui ont t fait dans la version temporaire et construira
			pour chaque rpertoire un guide qui controlera l'dition
			de la fusion.

			La prparation d'un fusion produit une srie de fichier
			intgre.dat dans la rvision destination. Elle n'a pas
			d'effet dans la rvision temporaire. Donc c'est un
			opration non dommageable.
		*/
		ufusion_prepare (user);
	}else if (key == u_version){
		projetp_chkver (user,true);
	}else if (key == u_commande){
		if (projetp_chkver (user,false)){
			/* #Spcification: menu / usager / commande
				Aprs qu'un projet ait t install dans
				l'environnement commun /kit/build, on peut
				en installer des parties dans l'environnement
				de travail de l'usager.

				On choisit une rvision, ensuite les projets
				sur lesquels on veut travailler.

				Un sous-rpertoire portant le numro
				de la rvision sera cr et les diffrents
				projets slectionns seront installs.
			*/
			PRJLISTE_GET g;
			projet_select (user,g);
		}
	}else if (key == u_commande_autre){
		if (projetp_chkver (user,false)){
			/* #Spcification: menu / usager / commande autre
				Lorsqu'on slectionne un projet pour installation
				dans notre environnement, on peut slectionner
				les projets qui ne font pas partie de la rvision.

				Cette possibilit permet de ressuciter des projets
				qui ont t limins. Cette qute vers le pass
				est gnralement une source de joie et de bonheur
				sans fin...
			*/
			PRJLISTE_GETALL g;
			projet_select (user,g);
		}
	}else if (key == u_fusion_edit){
		/* #Spcification: menu / usager / fusion
			Cet option prsente une liste de rpertoires
			o une intervention est requise.
			Pour chaque rpertoire l'usager obtient une liste
			des fichiers  combiner ou importer dans la
			rvision destination.
		*/
		#ifdef MISSING
			integre_do (user);
		#endif
	}else if (key == u_fusion_complexe){	
		/* #Spcification: menu / usager / fusion / prpare complexe
			On doit slectionner la rvision qui servira
			de base  l'intgration (BASE). Ensuite on
			slectionnera les rvisions qui seront fusionns (MODIFs).
			Pour chaque rvision, on devra associ une rvision
			rfrence (REFs).

			En gros, la fusion peut tre reprsent par l'quation.

			RESULTAT = BASE + (MODIFs - REFs)

			Le rsultat sera plac dans la rvision slectionn
			et l'dition se fera la. Il est prfrable de livrer
			pralablement. On pourra plus facilement dterminer la
			provenance d'un changement plus tard.
		*/
		bfusion_setup (user,1);
	}else if (key == c_livraison){
		/* #Spcification: menu / commun / livraison
			Aprs une fusion dans l'espace commun (/kit/build), on
			se retrouve gnralement avec une srie de changements
			qu'il faut rappatrier dans l'archive. On slectionnera
			gnralement tous les projets.

			La rvision reste alors disponible dans l'espace commun
			pour que les usagers prennent une copie de cette rvision.

			Il est prfrable de livrer les corrections avant que le
			premier usager prenne une copie.
		*/
		char path[MAXSIZ_PATH];
		user->getbld(path);
		if (projet_chkspace(user)==1
			&& version_setmain (user,path)){
			USERBUILD *build;
			{
				VERSION_DAT ver(user);
				const REVISION *rev = user->getrevdst();
				const char *vername = user->getversion();
				VERSION_ONE *one = ver.get (rev);
				if (one->isinteg()){
					build = new USERINTG (vername,rev,rev);
				}else{
					build = new USERBUILD (vername,rev);
				}
			}
			PRJLISTE_LIVREB g;
			projet_select (build,g);
			delete build;
		}
	}else if (key == c_compilation){
		if (projetp_chkver(user,1)){
			/* #Spcification: menu / Commun / Compilation
				Avant que les usagers puissent installer une
				rvision dans leur environnement, il faut
				qu'elle soit installe dans /kit/build et rebatie
				(compile  partir de 0).

				On slectionne une rvision et les sous-projets
				qu'on veut install.

				Bien qu'on puisse slectionner les projets sparment,
				il est souhaitable de tout installer, parce que
				l'opration de compilation risque de ne pas fonctionner.
			*/
			USERBUILD build(user->getversion(),user->getrevsrc());
			PRJLISTE_BUILD g;
			projet_select (&build,g);
		}		
	}else if (key == c_commande_autre){
		if (projetp_chkver(user,1)){
			/* #Spcification: menu / commun / commande autre
				Lorsqu'on slectionne un projet pour installation
				dans l'environnement commun, on peut slectionner
				les projets qui ne font pas partie de la rvision.

				Cette possibilit permet de ressuciter des projets
				qui ont t limins. Cette qute vers le pass
				est gnralement une source de joie et de bonheur
				sans fin...
			*/
			USERBUILD build(user->getversion(),user->getrevsrc());
			PRJLISTE_BUILDALL g;
			projet_select (&build,g);
		}
	}else if (key == c_diff){
		diff_select (user);
	}else if (key == c_fusion_prepare){
		if (projetp_chkver(user,1)){
			/* #Spcification: menu / commun / fusion / prpare
				On doit slectionner la rvision qui servira
				de base  l'intgration (BASE). Ensuite on
				slectionnera les rvisions qui seront fusionns (MODIFs).
				Pour chaque rvision, on devra associ une rvision
				rfrence (REFs).

				En gros, la fusion peut tre reprsent par l'quation.

				RESULTAT = BASE + (MODIFs - REFs)

				La rvision RESULTAT sera automatiquement alloue.
			*/
			USERBUILD build(user->getversion(),user->getrevsrc());
			bfusion_setup (&build,0);
		}
	}else if (key == c_fusion_edit){
		if (projetp_chkver(user,1)){
			// Mme spcification que: menu / usager / fusion / dition
			{
				USERBUILD build(user->getversion(),user->getrevsrc());
				//integre_do (&build);
			}
		}
	}else if (key == a_creation){
		/* #Spcification: menu / admin / cration
			On peut crer une nouvelle rvision simplement en
			renommant une ancienne. On peut alors crer un nouveau
			numro ainsi qu'un nouveau nom.

			On utilise ce mcanisme lorsqu'on livre un produit
			aux clients.

			Soit l'exemple suivant:

			A partir de la rvision 0.0, les programmeurs travaillent
			durant 1 an et produisent finalement la rvision 0.80.

			Cette version est envoye aux clients. Pendant qu'une
			partie de l'quipe de programmation sera occupe  corriger
			les bugs de dernire minutes, l'autre partie continuera
			 dvelopper sans rserve. Pour viter la confusion on
			crer deux arbres de dveloppement.

			#
			0.80 -- 0.81 ... 0.89 --+-- ... 0.95 -------+
			  |						|			        +-- 1.15
			  |						+-- 1.06 ... 1.14 --+
			  |						|
			1.0  --	1.01 ... 1.05 --+
			#

			Initialement 1.0 et 0.80 sont quivalent. En principe
			0.80 voluera peu en terme du nombre de fichiers sources
			modifi. Par contre, il y aura plusieurs rvisions cres,
			chacune correspondant  une version officielle envoye
			aux usager.

			1.0 de son cot sera chambarde pour incorporer 1,000,000
			de nouvelles options.

			On voit sur le schma que 1.06 et 1.15 seront des
			rvisions qui fusionnent les corrections de bugs de la
			version 0.80 aux nouveaux dveloppement de 1.0.
		*/
		{
			VERSION_DAT version(user);
			version.creation (user);
		}
	}else if (key == a_destruction){
		/* #Spcification: menu / admin / destruction
			Un jour :-), on pourra liminer une srie de rvision
			de l'archive. Ces rvisions disparaitront, mais pas
			ncessairement les fichiers qui les composent, puisque
			ces fichiers peuvent appartenir  plusieurs rvisions
			diffrentes.

			*** Pas encore oprationnel ***
		*/
		pasfait = true;
	}else if (key == a_menage){
		/* #Spcification: menu / admin / mnage
			Opration longue et pnible qui localisent tous les
			fichier de l'archive qui ne sont pas associs  aucune
			rvisions. Ces fichiers sont simplement effacs ou
			encore une liste est cre permettant de faire une
			copie de scurit et ensuite de les effacs.

			*** Pas encore oprationnel ***
		*/
		pasfait = true;
	}else if (key == a_importation){
		/* #Spcification: menu / admin / Importation
			Prsente la liste des toutes les rvisions qui ont
			t cre dans des pays lointain et placer dans
			/kit/livre. Seule les rvisions qui ne sont pas
			dj dans l'archive sont prsente. On les slectionne
			et elles sont immdiatement incorpores.
		*/
		if (projet_chkspace(user)==1){
			import (user);
		}
	}else{
		pasfait = true;
	}
	if (pasfait){
		xconf_error (MSG_U(E_PASFAIT
			,"Je suis dsol ... trs dsol\n"
			 "mais ce que vous me demand l\n"
			 "n'est pas encore disponible.\n"));
	}
}
#if 0
/*
	Obtient le path courant de l'usager avant d'appeler projetx.

	Ce programme (projetp) doit excuter dans un rpertoire spcifique
	o les fonts de GEM doivent se trouver. Le programme projetx note
	le rpertoire courant et l'inscrit dans un fichier portant le nom
	de l'usager avec l'extension "cwd".
*/
static int near projet_readcwd(
	const char *prog,
	char *cwd,
	int sizecwd)
{
	int ret = -1;
	char path[MAXSIZ_PATH];
	sprintf (path,"%s.cwd",prog);
	FILE *fin = fopen_err (path,"r",1);
	cwd[0] = '\0';
	if (fgets(cwd,sizecwd-1,fin)!=NULL){
		strip_end (cwd);
		ret = 0;
	}
	fclose (fin);
	return ret;
}
#endif

int main (int , char *[])
{
	etc_loadmsg();
	html_registerpath ("/usr/lib/projetx");
	const char *prog = prog_getid (true);
	if (projet_chkenv(prog)!=-1){
		DIALOG_MENU dia;
		u_version = MSG_U(M_VERSION,"Version");
		u_commande = MSG_U(M_COMMANDE,"Commande");
		u_livraison = MSG_U(M_LIVRAISON,"Livraison");
		u_menage = MSG_U(M_MENAGE,"Menage");
		u_fusion_simple = MSG_U(M_FUSIONSIMPLE,"Fusion simple");
		u_fusion_complexe = MSG_U(M_FUSIONCOMPLEXE,"Fusion complexe");
		u_fusion_edit = MSG_U(M_FUSIONEDIT,"dition de la fusion");
		u_commande_autre = MSG_U(M_COMMANDEAUTRE,"Commande autre");

		c_compilation = MSG_U(M_COMPILATION,"Compilation");
		c_livraison = MSG_U(M_C_LIVRAISON,"Livraison");
		c_fusion_prepare = MSG_U(M_C_FUSION_PREP,"Prpare fusion");
		c_fusion_edit = MSG_U(M_C_FUSION_EDIT,"dite la fusion");
		c_commande_autre = MSG_U(M_C_COMMANDE_AUTRE,"Commande autre");
		c_diff = MSG_U(M_C_DIFF,"Fichier patch");

		a_initarch = MSG_U(M_INITARCH,"Initialise l'archive");
		a_creation = MSG_U(M_CREATION,"Cration");
		a_destruction = MSG_U(M_DESTRUCTION,"Destruction");
		a_menage = MSG_U(M_A_MENAGE,"Mnage");
		a_importation = MSG_U(M_IMPORT,"Importation");

		static const char *opts[]={
			"-",	MSG_U(M_USAGER,"Usager"),
			"",		u_version,
			"",		u_commande,
			"",		u_livraison,
			"",		u_menage,
			"",		u_fusion_simple,
			"",		u_fusion_complexe,
			"",		u_fusion_edit,
			"",		u_commande_autre,
			"-",	MSG_U(M_COMMUN,"Commun"),
			"",		c_compilation,
			"",		c_livraison,
			"",		c_fusion_prepare,
			"",		c_fusion_edit,
			"",		c_commande_autre,
			"",		c_diff,
			"-",	MSG_U(M_ADMIN,"Admin"),
			"",		a_initarch,
			"",		a_creation,
			"",		a_destruction,
			"",		a_menage,
			"",		a_importation,
			NULL
		};
		dia.new_menuitems (opts);
		int nof = 0;
		while (1){
			MENU_STATUS code = dia.editmenu (
				MSG_U(T_PROJETX,"ProjetX")
				,""
				,help_nil
				,nof,0);
			if (code == MENU_QUIT || code == MENU_ESCAPE){
				break;
			}else{
				const char *sel = dia.getmenustr(nof);
				if (sel == a_initarch){
					/* #Spcification: menu / admin / initialise l'archive
						Ce menu ne sert qu'un fois pour crer une nouvelle archive.
						Il y aura cration des rpertoire et du minimum de fichier
						pour dmarrer l'archivage.
					*/
					arch_init();
				}else{
					user = userinfo_findenv (prog);
					if (user != NULL){
						projet_main (sel);
						delete user;
					}
				}
			}
		}
	}
	return 0;
}

