#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <misc.h>
#include "projetx.m"
#include "projet.h"

/*
	Renomme un rpertoire.
	Novell est capricieux. On ne peut renommer que des sous-rpertoires
	du rpertoire courant. Il faut donc changer de path
*/
static int userinf_rendir (
	const char *olddir,
	const char *newdir,
	PROJET_LOG *log)		// Affiche message confirmation si != NULL
{
	int ret = -1;
	// Dans le cas d'une intgration, il n'y a pas de renommage
	// de rpertoire. On conserve le mme numro de rvision
	if (strcmp(olddir,newdir)==0){
		ret = 0;
	}else if (file_type(olddir)!=1){
		if (log != NULL){
			log->printf ("%s n'existe pas: OK\n",olddir);
		}
	}else{
		char path[MAXSIZ_PATH];
		char odir[MAXSIZ_NAME];
		char ndir[MAXSIZ_NAME];
		path_splitlex (olddir,path,odir);
		path_splitlex (newdir,NULL,ndir);
		SAVEPATH save;
		if (log != NULL){
			log->printf ("pushdir :%s:\n",path);
		}
		if (path_pushdir (path,&save)!=-1){
			if (file_rename (odir,ndir) != -1) ret = 0;
			if (log != NULL){
				log->printf ("file_rename %d\n",ret);
			}
			path_popdir (&save);
		}
		if (log != NULL){
			log->printf ("%s -> %s (statut %d)\n",olddir,newdir,ret);
		}
		if (ret == -1){
			xconf_error (MSG_U(E_NORENAME
				,"Ne peut pas renommer le rpertoire\n%s\n    \n"
				 "%s")
				,olddir,newdir);
		}else{
			if (log != NULL) log->printf ("%s -> %s\n",olddir,newdir);
		}
	}
	return ret;
}

/*
	Transforme un USERINFO pour qu'il reprsente une nouvelle rvision.
	Optionnellement les rpertoires usagers seront renomms.
	On utilise cette fonction aprs un livraison. Le principe de la
	livraison est non destructif. L'usager livre les modifications
	dans l'archive. Les fichiers livrs sont effacs de sont environnement
	et le makefile.dat est modifi. Cette opration est rpte pour
	tous les projets livrs. Elle est ralise au niveau de la racine.

	Aprs la livraison, l'usager a entre les mains une nouvelle rvision
	qu'il peut a tout moment extraire de l'archive. Il faut donc
	renommer les rpertoires de l'usager.
*/
PUBLIC void USERINFO::chgenv(
	REVISION_TYPE revtype,
	int renomme,
	PROJET_LOG *log)		// Affiche message confirmation si != NULL
{
	// Prend une copie des paths avant la correction  la rvision
	char oldusrpath[MAXSIZ_PATH];
	char oldobjpath[MAXSIZ_PATH];
	makusrpath("",oldusrpath);
	makobjpath("",oldobjpath);

	REVISION newrev;
	setrevnxt(&newrev,revtype);
	rev = newrev;
	newrev.formatpath (revpath);

	// Calcul le nouveau path pour les sources
	char newusrpath[MAXSIZ_PATH];
	path_splitlex (oldusrpath,newusrpath,NULL);
	path_make (newusrpath,revpath,newusrpath);
	free (usrpath);
	usrpath = strdup_err (newusrpath);

	// Calcul nouveau path pour objets
	char newobjpath[MAXSIZ_PATH];
	path_splitlex (oldobjpath,newobjpath,NULL);
	path_make (newobjpath,revpath,newobjpath);
	free (objpath);
	objpath = strdup_err (newobjpath);

	if (renomme){
		userinf_rendir (oldusrpath,newusrpath,log);
		userinf_rendir (oldobjpath,newobjpath,log);
	}
}
/*
	Variation de USERINFO dcrivant une environnement pour compilation.

	Aprs avoir archiv, on veut mettre en service une rvision
	d'un projet, on doit recompiler tout. On prend simplement
	les projets  recompiler
	et on les place dans un sous-rpertoire de kit/build. Le nom du
	sous-rpertoire provient du numro de rvision de la version  rebatir.

	Dans chaque projet, on applique "make rebuild". Cette commande a deux
	effet. Elle produit des fichiers objets et librairies dans le rpertoire
	obj/OS/rvision et produit localement certains fichiers (Prototypes,
	fichiers d'inclusion mis  jour, dfinition de messages (*.m), etc...).

	De plus, les rpertoire ../h et ../doc sont peupler grace  cette
	commande. Ces rpertoires ne sont jamais archivs.
*/
PUBLIC USERBUILD::USERBUILD(
	const char *_version,		// Version majeure (symbolique)
	const REVISION *_rev)		// Rvision spcifique.
{
	strcpy (prog,prog_getid(true));
	noprog = prog_getnum(prog);
	strcpy (version,_version);
	_rev->formatpath (revpath);
	rev = *_rev;
	char path[MAXSIZ_PATH];
	build  = userinfo_chkenv("KITBUILD",KITBUILD);
	path_make (build,revpath,path);
	usrpath = strdup_err (path);
	findobj ();
	groupe  = userinfo_chkenv("KITGROUPE",KITGROUPE);
	ombre   = userinfo_chkenv("KITOMBRE",KITOMBRE);
	livre   = userinfo_chkenv("KITLIVRE",KITLIVRE);
}

/*
	Indique si l'environnement dcrit est celui utilise pour intrgration.
*/
PUBLIC VIRTUAL int USERBUILD::isbuild() const
{
	return 1;
}

/*
	Variation de USERBUILD dcrivant une environnement pour intgration.

	La diffrence entre un environnement USERBUILDet USERINTG, c'est que
	le numro de rvision qui sert a calculer les paths n'existe pas
	encore pour USERINTG. Voir setrevnxt()

	revbase agit comme source dans l'archive. C'est  partir de cette
	rvision que l'on extrait l'environnement de base dans kit/build.

	Toutefois, l'environnement dans kit/build sera plac dans un sous-
	rpertoire associ  la rvision revdest. Cette rvision n'existe pas
	encore dans l'archive.
*/
PUBLIC USERINTG::USERINTG(
	const char *_version,		// Version majeure (symbolique)
	const REVISION *_revbase,	// Rvision de base  extraire.
	const REVISION *_revdest)	// Rvision qui contiendra l'intgration.
								// C'est aussi la destination pour copier.
	: USERBUILD(_version,_revdest)
{
	revbase = _revbase;
}

/*
	Obtient la rvision de provenance dans l'archive.
*/
PUBLIC const REVISION *USERINTG::getrevsrc() const
{
	return &revbase;
}

/*
	Dtermine le prochaine rvision pour la livraison.
	Dans le cas d'une integration, le nouveau numro est le meme, car
	le numro rev n'existe pas encore dans l'archive.
*/
PUBLIC void USERINTG::setrevnxt(
	REVISION *newrev,
	REVISION_TYPE)	// revtype : L'intgration produit toujours une rvision
					// d'intgration, donc on ignore le type de rvision.
	const
{
	*newrev = rev;
}

/*
	Enregistre un message d'erreur avec description de l'environnement
	dans error.log.
*/
PUBLIC void USERINFO::logerr (
	const char *ctl,
	...) const
{
	/* #Spcification: opration / erreurs critique / log
		Lorsqu'un anomalie est detecte durant un opration sur l'archive,
		un message est affich  l'cran et une description plus complte
		est plac dans le fichier error.log.

		En consultant ce fichier de temps en temps, on pourra vrifier
		que l'archive et ses nombreux fans vivent dans le bonheur.
	*/
	FILE *fout = log_fopen ("error.log",1);
	va_list list;
	va_start (list,ctl);
	vfprintf (fout,ctl,list);
	va_end   (list);
	// Affiche le plus de dtail possible.
	char cwd[MAXSIZ_PATH];
	if (path_getcwd(cwd,sizeof(cwd)) != -1){
		fprintf (fout,"\tCWD=%s\n",cwd);
	}else{
		fprintf (fout,"\tCWD ne peut pas etre determiner\n");
	}
	fprintf (fout,"\tUsager = %s(%d)\n",prog,noprog);
	fprintf (fout,"\tVersion/Revision = %s/%s\n",version,revpath);
	fprintf (fout,"\tUsrpath = %s\n",usrpath);
	fprintf (fout,"\tObjpath = %s\n",objpath);
	fprintf (fout,"\tOmbre   = %s\n",ombre);
	fprintf (fout,"\tBuild   = %s\n",build);
	fprintf (fout,"\tLivre   = %s\n",livre);
	fprintf (fout,"\terrno   = %d (%s)\n",errno,strerror(errno));
	fclose (fout);
}

