#include <stdlib.h>
#include <string.h>
#include <misc.h>
#include "projetx.m"
#include "projet.h"
/*
	Dtermine la valeur d'une paramtre de configuration  partir de getenv().
	Si pas dans l'environnement, utilise defval.

	Retourne une copie de la slection via strdup_err().
*/
char *userinfo_chkenv(
	const char *varname,	// Nom de la variable
	const char *defval)		// Valeur par dfaut
{
	defval = macrodir_getval (varname,defval);
	const char *pt = getenv (varname);
	if (pt != NULL) defval = pt;
	return strdup_err (defval);
}
/*
	Slectionne une version de projet  partir du path courant.
	Si ne peut pas slectionner, demande interactivement.
	Retourne la structure de controle. Ou NULL si abandon.

	Valide aussi l'environnement usager. Si trouve quelques chose de
	pas catholique, gnre un message d'erreur et retourne NULL.
*/
USERINFO *userinfo_findenv (const char *prog)
{
	USERINFO *user = NULL;
	const char *home = getenv("HOME");
	home = macrodir_getval ("HOME",home);
	if (home == NULL){
		xconf_error (MSG_U(E_NOHOME,"Variable HOME non dfinie"));
	}else{
		/* #Spcification: Archivage / Rpertoires / Path par dfaut.
			Par dfaut, les rpertoires d'archive ont un emplacement absolu.
			On peut control chacun via une variable d'environnement.

			Rpertoire		DOS				UNIX				Variable

			/kit/ombre		n:\kit\ombre	/kit/ombre			KITOMBRE
			/kit/build		n:\kit\build	/kit/build			KITBUILD
			/kit/groupe		n:\kit\groupe	/kit/groupe			KITGROUPE
			/kitobj			n:\kitobj		/home/obj/build/gnu
			/kit/livre		n:\kit\livre	/kit/livre			KITLIVRE
			/kit/sync		n:\kit\sync		/kit/sync			KITSYNC
		*/
		const char *groupe = userinfo_chkenv("KITGROUPE",KITGROUPE);
		const char *ombre  = userinfo_chkenv("KITOMBRE",KITOMBRE);
		#ifdef MISSING
			const char *build  = userinfo_chkenv("KITBUILD",KITBUILD);
		#endif
		const char *livre  = userinfo_chkenv("KITLIVRE",KITLIVRE);
		const char *sync   = userinfo_chkenv("KITSYNC",KITSYNC);

		if (file_type(groupe)!=1){
			xconf_error (MSG_U(E_NOKITGROUPE
				,"Ne peut localiser le rpertoire d'archive\n%s")
				,groupe);
		}else if (file_type(ombre)!=1){
			xconf_error (MSG_U(E_NOKITOMBRE
				,"Ne peut localiser le rpertoire\n"
				 "d'extraction %s")
				,ombre);
		#ifdef MISSING
			}else if (file_type(build)!=1){
				xconf_error (MSG_U(E_NOKITBUILD
					,"Ne peut localiser le rpertoire d'intgration\n%s")
					,build);
		#endif
		}else if (file_type(livre)!=1){
			xconf_error (MSG_U(E_NOKITLIVRE
				,"Ne peut localiser le rpertoire\n"
				 "de livraison\n%s")
				,livre);
		}else if (file_type(sync)!=1){
			xconf_error (MSG_U(E_NOKITSYNC
				,"Ne peut localiser le rpertoire\n"
				 "de synchronisation\n%s")
				,sync);
		}else{
			// Tente de dterminer la version  partir du rpertoire
			user = new USERINFO (prog,"",home,groupe,ombre,livre,sync);
			int ok = 0;
			char cwd[MAXSIZ_PATH];
			if (path_getcwd(cwd,sizeof(cwd)) != -1){
				int len = strlen(home);
				ok = 1;
				if (strncasecmp(home,cwd,len)==0
					&& path_issep(cwd[len])){
					// On est dans le rpertoire usager.
					// il faut tre dans le projet
					char *verprj = cwd + len + 1;
					// La version c'est le premier rpertoire sous home
					char *pt = verprj;
					while (*pt != '\0' && !path_issep(*pt)) pt++;
					*pt = '\0';
					VERSION_DAT version(user);
					VERSION_ONE *one = version.select(verprj,prog);
					if (one != NULL){
						user->setversion (one);
						ok = 2;
					}
				}
			}
			if (!ok){
				if (version_setmain (user,NULL) == 0){
					delete user;
					user = NULL;
				}
			}
		}
	}
	return user;
}

