/*
	reconnaissance syntaxique et affichage des prototypes
*/

#include <stdio.h>
#include <string.h>
#include "assert.h"
#include <lexc.h>
#include "proto.h"

/*
	Affiche tous les arguments d'une dclaration. Rcursif.
	retourne le nombre de caractere imprim sur la ligne.
*/
static int prtall (
	TOKEN_LIST *tokl,
	int nbout,		/* nombre de caractere deja imprime sur la ligne */
	int outmode,
	int oneparm)	/* Un paramtre par ligne ? */
{
    TOKEN *lasttok=NULL;
    do {
		TOKEN *tok = tokrec_get (tokl);
		TOK_TYPE tok_type = tok->type;
		char *txt  = token_txt  (tok);
		if (nbout + strlen(txt) > 78){ 
			protof (outmode,"\n\t");
			nbout = 8;
		}
		if (token_needsep(tok,lasttok))	nbout+=	protof (outmode," ");
        lasttok = tok;
		static int defval_mode=0;
		if (tok_type == TOK_KEYWORD && tok->type2.keyword == KEY_P_DEFVAL){
			defval_mode = 1;
			nbout += protof	(outmode,"=");
		}else if (defval_mode > 0){
			if (tok_type == TOK_CLSPAR){
				defval_mode--;
				if (defval_mode > 1){
					nbout += protof(outmode,")");
				}else{
					defval_mode = 0;
				}
			}else if (tok_type == TOK_OPNPAR){
				defval_mode++;
				if (defval_mode > 2) nbout += protof(outmode,"(");
			}else{
				nbout += protof	(outmode,"%s",txt);
			}
		}else{
			nbout += protof	(outmode,"%s",txt);
		}
		if (tok_type == TOK_VIRGULE){
			if (oneparm) {
				protof (outmode,"\n\t");
				nbout = 8;
			}
		}else if (tok_type == TOK_OPNPAR && tokrec_descend(tokl)!=-1){
			nbout =	prtall (tokl,nbout,outmode,0);
			tokrec_monte  (tokl);
		}
	}while (tokrec_next (tokl)!=-1);
	return nbout;
}
static int	 outmode;
/*
	Retourne nombre de caractre imprime
*/

static int localprt	(char *ctl,...)
{
	int nbcar;
	va_list	list;
	va_start (list,ctl);
	nbcar = protof_vprint (outmode,ctl,list);
	va_end (list);
	return (nbcar);
}

static int cplus_def=MODE_PRIVATE;
/*
	Enregistre l'accs par dfaut des fonctions membres C++
*/
void cproto_setcplusdef (int def)
{
	cplus_def = def;
}

/*
	Gnre les prototypes avec et sans documentation.
*/
void cproto_new	(
	TOKEN_LIST *tokl,
	int cplus,		/* Fonction membre C++ */
	int ret_fct,	/* Fonction qui retourne un pointeur de fonction */
	int operator_fct,/* Fonction operator()(...) */
	int extern_C)	/* Precede prototype par extern "C" */
{
	int tmp_operator_fct = operator_fct;
	char nom[100];
	TOKEN *token;
	int  oneparm = tokrec_getlentxt (tokl) > 78;
	extern_C = !cplus && extern_C;
	nom[0] = '\0';
	tokrec_inilec (tokl);
	token =	tokrec_get (tokl);
	/*
		Reconnait le type de fonction: static, export, etc ...
		La fonction d'output duplicate le rsultat dans les diffrents
		fichiers qui accepte ce type.
	*/

	outmode	= cplus ? cplus_def : MODE_ORDIN;
	if (token->type == TOK_KEYWORD){
		if (cplus){
			if (token->type2.keyword == KEY_PRIVATE){
				outmode	= MODE_PRIVATE;
			}else if (token->type2.keyword == KEY_PROTECTED){
				outmode	= MODE_PROTECTED;
			}else if (token->type2.keyword == KEY_PUBLIC){
				outmode	= MODE_PUBLIC;
			}
		}else if (token->type2.keyword == KEY_STATIC){
			outmode = MODE_STATIC;
		}else if (token->type2.keyword == KEY_EXPORT
			|| token->type2.keyword == KEY__EXPORT){
			outmode	= MODE_EXPORT;
		}
    }
	{
		TOKEN *lasttok=NULL;
		int cplusfind = 0;
		int  nbout=0;
		int nbskippar = ret_fct ? 1 : 0;
		char out[100];
		if (extern_C){
			strcpy (out,"extern \"C\" ");
		}else{
			out[0] = '\0';
		}
		/*
			Collecte la dclaration du type dans out.
			Arrte  la parenthse qui introduit les arguments.
		*/
		while(1){
			char *txt;
			int opnpar;
            token=tokrec_get(tokl);
			assert (token!=NULL);
			txt = token_txt(token);
			if (strcmp(txt,"main")==0) return;
			if (token_needsep(token,lasttok)) strcat (out," ");
			if (token->type == TOK_SCOPE){
				cplusfind = 1;
	            strcat (out,txt);
			}else{
	            strcat (out,txt);
			}
			opnpar = token->type == TOK_OPNPAR;
			if (opnpar && tmp_operator_fct){
				tmp_operator_fct=0;
				opnpar = 0;
			}
			if (opnpar){
				if (nbskippar > 0){
					tokrec_descend(tokl);
					nbskippar--;
				}else{
					break;
				}
			}else{
				if (cplusfind){
					strcat (nom,txt);
				}else{
					strcpy (nom,txt);
				}
				lasttok = token;
				tokrec_next (tokl);
			}
        }
		nbout = protof (outmode,"%s",out);
        {
			int ok = tokrec_descend (tokl);
			assert (ok!=-1);
			token =	tokrec_get(tokl);
			assert (token!=NULL);
			assert (token->type!=TOK_CLSPAR);
			nbout = prtall (tokl,nbout,outmode,oneparm);
			tokrec_monte (tokl);
			if (tokrec_next (tokl) != -1){
				/*
					Il y a le mot cl const utilis en C++
					Ou c'est une fonction qui retourne un pointeur
					de fonction et c'est son prototype
				*/
				nbout = prtall (tokl,nbout,outmode,oneparm);
				if (ret_fct){
					tokrec_monte(tokl);
					if (tokrec_next(tokl)!=-1){
						prtall (tokl,nbout,outmode,oneparm);
					}
				}
			}
			protof (outmode,";\n");
		}
	}
	{
		protof_nomfct (outmode|MODE_INDEX,nom);
		outmode |= MODE_COMMENT;
		protof_nomfct (outmode,nom);
		cproto_format (tokl,localprt,ret_fct,operator_fct,extern_C);
	}
}
/*
	Trouve une variable dans une liste de declaration
	varl ne contient pas le type generique
	retourne la position dans la liste ou -1

	ATTENTION : Dans une varl sans type generique, le premier TOK_ID
				rencontre est le nom de la variable.
*/
static int cproto_findvar (
	TOKEN *var,
	TOKEN_LIST varl[],
	int nbvar)
{
	int i;
	for (i=0; i<nbvar; i++){
		TOKEN *tok;
		TOKEN_LIST *list = &varl[i];
		tokrec_inilec (list);
		while ((tok = tokrec_get (list)) != NULL){
			if (tok->type == TOK_ID){ 
				if (strcmp (token_txt(var),token_txt(tok))==0) return(i);
				break;
			}else if (tok->type == TOK_OPNPAR){
				tokrec_descend(list);
			}else{
				tokrec_next(list);
			}
		}
	}
	return (-1);
}
/*
	A partir de la declaration ancience, reformatte 
*/
static void cproto_cnvnew (
	TOKEN_LIST *decl,
	TOKEN_LIST typel[],
	TOKEN_LIST varl[],
	int nbvar,
	TOKEN_LIST *newt)
{
	TOKEN *tok;
	int ret_fct = 0;

	tokrec_init (newt);
	tokrec_inilec (decl);
	while ((tok=tokrec_get(decl))!=NULL && tok->type != TOK_OPNPAR){
		tokrec_add (newt,tok);
		tokrec_next (decl);
	}
	{
		int ok;
		assert (tok != NULL);
		tokrec_add (newt,tok);
		tokrec_newsub (newt);
		ok = tokrec_descend (decl);
		assert (ok!=-1);
		if ((tok=tokrec_get(decl))!=NULL && tok->type == TOK_STAR){
			ret_fct = 1;
			/*
				Fonction qui retourne pointeur de fonction
			*/
			while ((tok=tokrec_get(decl))!=NULL && tok->type != TOK_OPNPAR){
				tokrec_add (newt,tok);
				tokrec_next (decl);
			}
			assert (tok != NULL);
			tokrec_add (newt,tok);
			tokrec_newsub (newt);
			ok = tokrec_descend (decl);
			assert (ok!=-1);
		}
		while ((tok=tokrec_get(decl))!=NULL && tok->type != TOK_CLSPAR){
			tokrec_next (decl);
			if (tok->type == TOK_VIRGULE){
				tokrec_add (newt,tok);
			}else{
				int novar;
				assert (tok->type == TOK_ID);
				novar = cproto_findvar (tok,varl,nbvar);
				if (novar!=-1){
					tokrec_addseq (newt,&typel[novar]);
					tokrec_addseq (newt,&varl[novar]);
				}else{
					char out[200];
					tokrec_add (newt,tok);
					tokrec_frmtxt (newt,out);
					erreur_signal (
						"paramtre %s sans dclaration : 'int' gnr\n\t%s...\n"
						,"Parameter %s without declaration : 'int' used\n\t%s...\n"
						,token_txt(tok),out);
				}
			}
		}
		assert (tok!=NULL);
		tokrec_add (newt,tok);
		if (ret_fct){
			/*
				Copie le reste de la dclaration si fonction qui retourne
				pointeur de fonction
			*/
			tokrec_endsub (newt);
			tokrec_monte  (decl);
			tokrec_next   (decl);
			tokrec_addend (newt,decl);
			tokrec_endsub (newt);
			tokrec_monte  (decl);
			tokrec_next   (decl);
			tokrec_addend (newt,decl);
		}
	}
}
/*
	interprete une declaration avec ancienne syntaxe et
	produit un prototype moderne
	Retourne -1 si erreur, 0 si ok
*/
int cproto_old (
	TOKEN_LIST *tokl,
	TOKEN      *tok)
{
	TOKEN_LIST typel[20];
	TOKEN_LIST varl[20];
	int novar = 0;
	int ok = 0;

	while (ok != -1 && tok->type != TOK_OPNBRACE){
		if (novar>=20){
			int nol = token_getnoline();
			erreur (
				"Trop de paramtres ligne %d\n"
				,"Too many parameters on line %d\n",nol);
			ok = -1;
		}else{
			novar += cproto_recdecl (&typel[novar],&varl[novar],tok);
			ok = token_get (tok);
		}
	}
	if (ok == 0){
		TOKEN_LIST newt;
		cproto_cnvnew (tokl,typel,varl,novar,&newt);
		tokrec_free   (tokl);
		*tokl = newt;
	}
	while (novar--){
		tokrec_free (&typel[novar]);
		tokrec_free (&varl[novar]);
	}
	return ok;
}
