/* permet de concatener des fichier *.nar et d'inserer un path */
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <tool.h>
#include <etc.h>
#include <misc.h>
#include "proto.m"

#define EXPNAR_VERSION	1
#define EXPNAR_RELEASE	2

void help_link()
{
	dir_getlist ("","",1,NULL,0);
}

static void near usage (void)
{
	fprintf (stderr,MSG_U(I_EXPNARUSAGE,
		"expnar [-s] fichier_destination fichier_nar [ fichier_nar ]\n"
		"expnar @fichier_arguments\n"
		"\n"
		"       Concatne plusieurs fichiers nar dans un seul\n"
		"       en insrant le nom du fichier dans chacune des\n"
		"       dfinition qu'il fournit.\n"
		"\n"
		"       Ne traite pas deux arguments possdant le mme\n"
		"       nom de base.\n"
		"       Ex: expnar resultat xx/ff.nar yy/ff.nar\n"
		"           Seul le premier ff.nar sera trait.\n"
		"\n"
		" -s : Pas de messages si un fichier_nar est manquant.\n"));
}

/*
	insert une chaine dans une autre et recalcule le byte de dimension
*/
static void expnar_insert (char *str, const char *insert)
{
	char *x = strstr (str,"! ");
	if (x == NULL){
		fprintf (stderr,MSG_B(E_IVLDFORM
			,"Format invalide:\n%s\n","Invalid format:\n%s\n"),str);
	}else{
		char tmp[200];
		str[0] += strlen(insert);
		x += 2;
		strcpy (tmp,x);
		x = stpcpy (x,insert);
		strcpy (x,tmp);
	}
}
/*
	Copie un fichier nar dans fout, en insrant son nom dans chaque ligne
	Retourne -1 si erreur.
*/
static int near expnar_copy (const char *fname, FILE *fout, int tolerant)
{
	FILE *fin = vfopen_err (fname,"r",tolerant ? 0 : 1);
	int ret = -1;
	if (fin != NULL){
		char buf[200];
		char base[MAXSIZ_NAME];
		printf ("-> %s\n",fname);
		file_baseext (fname,base,NULL);
		strcat (base,"/");
		while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
			strip_end (buf);
			expnar_insert (buf,base);
#if	defined(MSDOS) || defined(OS2)
			fprintf (fout,"%s\n",buf);
#else
			fprintf (fout,"%s\r\n",buf);
#endif
		}
		fclose (fin);
		ret = 0;
	}
	return ret;
}
/*
	Fusionne plusieurs fichiers nar dans un seul
*/
static void near expnar_merge (
	const char *dest,
	const char *src[],
	int nbsrc,
	int tolerant)
{
	int i;
	FILE *fout = fopen_err (dest,"w",1);
	char *deja[100];	/* Retient les fichiers dj traits */
	assert (nbsrc<100);
	for (i=0; i<nbsrc; i++){
		char nom[MAXSIZ_NAME];
		int j;
		int saute = 0;
		path_splitlex (src[i],NULL,nom);
		for (j=0; j<i; j++){
			if (strcmp(nom,deja[j])==0){
				saute = 1;	/* On a dj traits un fichier avec le mme */
							/* nom. On le fait pas deux fois */
				break;
			}
		}
		if (saute
			|| expnar_copy (src[i],fout,tolerant) == -1){
			nom[0] = '\0';
		}
		deja[i] = strdup_err (nom,1);
	}
	tbstr_free (deja,nbsrc);
	fclose (fout);
}

int main (int argc, char *argv0[])
{
	int ret = -1;
	etc_loadmsg();
	ARGP argp[26];
	char *argv[100];
	argc = anlparm_indir(argc,argv0,argv,100,argp,"s");
	if (argc < 3){
		usage();
	}else{
		expnar_merge (argv[1],(const char**)argv+2,argc-2
			,argp['s'-'a'].ptr != NULL);
		ret = 0;
	}
	return ret;
}


