/*
	Programme qui mule la commande copy de DOS avec rpertoire virtuel
*/
#include <assert.h>
#include <string.h>
#include <etc.h>
#include "ostool.h"
#include "commun.h"
#include <basio.h>

#define PCOPY_VERSION	1
#define PCOPY_RELEASE	7

unsigned int _stklen = 20000;

static void near usage(void)
{
	static const char *tbf[]={
		"pcopy [-s] [path/]fichiers ... [path/]destination",
		"copy avec rpertoire virtuel.",
		"\n",
		"-s   : Copie silencieuse.",
		"       Par dfaut, affiche un message pour chaque",
		"       fichier copi.",
		NULL
	};
	version_usagef (tbf,tbf,"pcopy","pcopy"
#	if		defined(VERSION_SHARE)
		,&version_share
#	elif	defined(VERSION_PROD)
		,&version_prod
#	elif	defined(VERSION_DEV)
		,&version_beta
#	elif	defined(VERSION_ESSAI)
		,&version_alpha
#	endif
		,PCOPY_VERSION,PCOPY_RELEASE
		,1992,0,0
		,NULL,NULL,NULL);
}
static char silent=0;

static int near pcopy_do (
	const char *src,
	const char *dst,
	int dst_is_dir,		// La destination est un rpertoire
	int ncopy)			/* Utilise la commande de Novell pour */
						/* Meilleure conservation des permissions (NFS) */
{
	char pathdst[MAXSIZ_PATH];
	char pathsrc[MAXSIZ_PATH];
	int ret = -1;
	if (dst_is_dir){
		char nom[MAXSIZ_NAME];
		path_splitlex (src,NULL,nom);
		path_make (dst,nom,pathdst);
		dst = pathdst;
	}
	if (vdir_exist (src,pathsrc) == 0){
		error_bprintf ("Le fichier %s n'existe pas\n"
				,"File %s does not exist\n",src);
	}else{
		if (!silent) printf ("%s -> %s\n",pathsrc,dst);
		if (path_cmp(pathsrc,dst)==0){
			error_bprintf ("Ne peut pas copier %s sur lui-mme\n"
				,"Can't copy %s over itself\n",pathsrc);
		}else{
			int nb_copy = 0;
			if (ncopy){
				if (system_f ("ncopy %s %s",pathsrc,dst)==0){
					nb_copy = 1;
				}
			}else{
				nb_copy = file_copyp (pathsrc,dst);
			}
			if (nb_copy != 1){
				error_bprintf ("Ne peut pas copier correctement\n"
					,"Can't complete copy correctly\n");
			}else{
				ret = 0;
			}
		}
	}
	return ret;
}
static int near pcopy_dolist (
	const char *wildsrc,
	const char *dest,
	int dest_is_dir,
	int ncopy)			/* Utilise la commande de Novell pour */
						/* Meilleure conservation des permissions (NFS) */
{
	int ret = -1;
	if (!dest_is_dir){
		error_bprintf ("%s doit tre un rpertoire\n"
			,"%s must be a directory\n",dest);
	}else{
		char *tb[1000];
		int nb = vdir_getlistp (wildsrc,tb,1000);
		ret = 0;
		for (int i=0; i<nb && ret != -1; i++){
			ret = pcopy_do (tb[i],dest,dest_is_dir,ncopy);
		}
		tbstr_free (tb,nb);
	}
	return ret;
}

int main (int argc, char *argv0[])
{
	int ret = -1;
	ARGP argp[26];
	char *argv[300];
	argc = anlparm_indir (argc,argv0,argv,300,argp,"s");
	if (argc < 3){
		usage ();
	}else{
		const char *dest = argv[--argc];
		int dest_is_dir = (int)(file_type(dest)==1);
		silent = argp['s'-'a'].ptr != NULL;
		if (argc > 2 && !dest_is_dir){
			error_bprintf ("%s doit tre un rpertoire\n"
				,"%s must be a directory\n",dest);
		}else{
			int ncopy = 0;
			#if 0
				char path[MAXSIZ_PATH];
				int ncopy = file_findexe ("ncopy",path) != -1;
			#endif
			ret = 0;
			for (int i=1; i<argc && ret != -1; i++){
				const char *src = argv[i];
				if (file_iswild(src)){
					ret = pcopy_dolist(src,dest,dest_is_dir,ncopy);
				}else if (vdir_ftype(src,NULL) == 1){
					char psrc[MAXSIZ_PATH];
					path_make (src,WILD_ALLFILE,psrc);
					ret = pcopy_dolist(psrc,dest,dest_is_dir,ncopy);
				}else{
					ret = pcopy_do (src,dest,dest_is_dir,ncopy);
				}
			}
		}
	}
	return ret;
}

