#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>
#include <string.h>
#include <utime.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <misc.h>
#include "tool.h"
#include "../common/commun.h"

/*
	Change le propritaire d'un fichier.
	Retourne -1 si erreur. Gnralement, seul root peut faire ca.
	Si user == -1, choisit usager courant, si group == -1, choisit group
	courant.

	Fonction spcifique  UNIX.
*/
export int file_chown (const char *fname, int user, int group)
{
	if (user == -1) user = getuid();
	if (group == -1) group = getgid();
	return chown (fname,user,group);
}
/*
	Copie un fichier dans un autre en conservant la mme date
	Retourne 0 ou -1 si erreur
	Retourne 1 si copie ok, mais la date n'est pas change.
*/	

static int near copy (
	const char *dirsrc,		/* Rpertoire source */
	const char *dirdst,		/* rpertoire destination */
	const char *namesrc,	/* Nom du fichier source ou wildcard*/
	const char *namedst)	/* nom du fichier destination ou NULL */
{
	int ok = -1;
	char pathsrc[MAXSIZ_PATH];
	char pathdst[MAXSIZ_PATH];
	path_make (dirsrc,namesrc,pathsrc);
	path_make (dirdst,namedst,pathdst);
#if 0
	consio_printf ("copy :%s: -> :%s:\n",pathsrc,pathdst);
	consio_keywait();
#elif defined(TEST)
	printf ("copy :%s: -> :%s:\n",pathsrc,pathdst);
#endif
	{
		FILE *fin = fopen (pathsrc,"r");
		if (fin!=NULL){
			if (file_writeok(pathdst)){
				FILE *fout = fopen (pathdst,"w");
				if (fout != NULL){
					int bufsiz = 20000;
					int nb;
					char *buf;
					while ((buf=(char*)malloc (bufsiz))==NULL){
						assert (bufsiz != 0);
						bufsiz /=2;
					}
					ok = 0;
					while ((nb=fread(buf,1,bufsiz,fin))>0){
						if((int)fwrite (buf,1,nb,fout)!=nb){
							ok = -1;
							break;
						}
					}
					free (buf);
					fflush(fout);
					if (ok != -1){
						struct utimbuf timbuf;
						struct stat	stbuf;
						stat (pathsrc,&stbuf);
						timbuf.modtime = stbuf.st_mtime;
						timbuf.actime = stbuf.st_atime;
						ok = utime (pathdst,&timbuf);
						if (ok == -1) ok = 1;
						fchmod (fileno(fout),stbuf.st_mode);
#if defined(TEST)
						extern int errno;
						printf ("Change date %d [%d]\n",ok,errno);
#endif
					}
					fclose (fout);
					file_chown (pathdst,-1,-1);
				}
			}
			fclose (fin);
		}
	}	
	return ok;
}

/*
	Copie un ou une srie de fichiers  partir de path dcomposs
	Retourne nombre de fichier copi si ok, -1 si erreur
*/	

export int file_copy (
	const char *dirsrc,		/* Rpertoire source */
	const char *dirdst,		/* rpertoire destination */
	const char *namesrc,	/* Nom du fichier source ou wildcard*/
	const char *namedst)	/* nom du fichier destination ou NULL */
{
	int ok = -1;
	if (file_type(dirsrc)==1 && file_type(dirdst)==1){
		if (file_iswild(namesrc)){
			char *tbstr[400];
			int nbstr = dir_getlist (dirsrc,namesrc,0,tbstr,400);
			int i;
			assert (namedst == NULL);
			for (i=0; i<nbstr; i++){
				char name[20];
				path_splitlex (tbstr[i],NULL,name);
				ok = copy (dirsrc,dirdst,name,name);
				if (ok == -1) break;
			}			
			tbstr_free (tbstr,nbstr);
			if (ok != -1) ok = nbstr;
		}else{
			if (namedst == NULL) namedst = namesrc;
			ok = copy (dirsrc,dirdst,namesrc,namedst);
			if (ok != -1) ok = 1;
		}
	}	
	return (ok);
}
/*
	Copie un ou une srie de fichiers a partir de deux paths
	Retourne nombre de fichier copi si ok, -1 si erreur
*/	

export int file_copyp (
	const char *pathsrc,		/* Rpertoire source */
	const char *pathdst)
{
	char paths[MAXSIZ_PATH];
	char files[MAXSIZ_NAME];
	char pathd[MAXSIZ_PATH];
	char filed[MAXSIZ_NAME];
	char *ptfiled = filed;
	int ret = 0;
	if (file_iswild(pathsrc)){
		path_splitlex(pathsrc,paths,files);
	}else{
		int types = file_type(pathsrc);
		if (types == 1){
			strcpy (paths,pathsrc);
			strcpy (files,"*");
		}else if (types == 0){
			path_splitlex (pathsrc,paths,files);
		}else{
			ret = -1;
		}
	}
	if (ret == 0){
		if (file_iswild(pathdst)){
			ret = -1;
		}else{
			int typed = file_type(pathdst);
			if (typed == 1){
				strcpy (pathd,pathdst);
				ptfiled = NULL;
			}else{
				path_splitlex (pathdst,pathd,filed);
			}
		}
	}
	if (ret == 0){
		ret = file_copy (paths,pathd,files,ptfiled);
	}
	return ret;			
}

#ifdef TEST

int main (int, char *[])
{
	file_copyp ("file6.c","/tmp/file6.c");
	exit (-1);

	file_copy ("../tool","/tmp","file6.c","toto");
	file_copyp ("../tool/file6.c","/tmp/toto.1");
	file_copyp ("../tool/file6.c","/tmp");
	system ("sh");
	file_copy ("../tool","/tmp","file6.c",NULL);
	system ("sh");
	file_copy ("../tool","/tmp","*.c",NULL);
	system ("sh");
	file_unlink ("/tmp/*.c");
	file_unlink ("/tmp/toto*");
	return 0;
}

#endif

