#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "tool.h"

#ifdef TEST
#define	testprt(x) printf(x)
#else
#define	testprt(x)
#endif


/*
	Obtient repertoire courant
	Retourne -1 si trop long ou erreur, 0 si ok
*/
export int path_getcwd (char *path,int buflen)
{
	int	ok = -1;
	FILEINFO info;
	static int lastinode=-1;
	static int lastmajor=-1;
	static int lastminor=-1;
	static char	lastpath[MAXSIZ_PATH];
	file_statinfo (".",&info);
	if (info.inode != lastinode
		|| info.device.major_num != lastmajor
		|| info.device.minor_num != lastminor){
		testprt	("Appelle getcwd\n");
		if (getcwd(path,buflen)	!= NULL){
			strcpy (lastpath,path);
			lastinode =	info.inode;
			lastmajor = info.device.major_num;
			lastminor = info.device.minor_num;
			ok = 0;
		}
	}else if ((int)strlen(lastpath)<buflen){
		testprt	("Utilise vieux getcwd\n");
		strcpy (path,lastpath);
		ok = 0;
	}
	return (ok);
}

/*
	indique si un path est la racine
	Retourne 1 si oui, 0 sinon
*/
export int path_isroot (const char *path)
{
	int ok = 0;
	if (strcmp(path,"/")==0){
		ok = 1;
	}
	return (ok);
}
/*
	change le path courant
	retourne -1	si erreur, 0 si	ok
	newpath	peut etre une chaine vide ("").
*/
export int path_chdir (const char *newpath)
{
	int ret = 0;
	if (*newpath != '\0'){
		char buf[MAXSIZ_PATH];
		path_stripsep (newpath,buf);
		if (chdir(buf) == -1) ret =	-1;
	}
	return (ret);
}

/*
	Change de rpertoire courant en	prservant le rpertoire prsent
	Si le path contient un disque, change de disque aussi
	Retourne 0 si ok, -1 si	erreur
	savepath pourra tre utilis avec path_popdir() pour annuler
	l'opration
	Il ne devrait pas y avoir de path_chdir entre le push et le pop
*/

export int path_pushdir (const char *path, SAVEPATH *savepath)
{
	int ok = 0;

	if (path_getcwd	(savepath->path,MAXSIZ_PATH) !=	0
		|| path_chdir  (path) != 0){
		ok = -1;
	}
	return (ok);
}
/*
	Annule l'effet de path_pushdir en revenant au rpertoire de dpart
	Change de disque aussi
	Retourne -1 si erreur, 0 si ok
*/

export int path_popdir (SAVEPATH *savepath)
{
	int	ok = 0;
	if (path_chdir (savepath->path)==-1) ok	= -1;
	return (ok);
}	
#ifdef TEST

static void show (void)
{
	char buf[MAXSIZ_PATH];
	int	i;
	for	(i=0; i<3; i++)	path_getcwd	(buf,sizeof(buf)-1);
	printf ("cwd = %s\n",buf);
}

static void tstmak (const char *base)
{
	char fp[MAXSIZ_PATH];
	path_make (base,"toto",fp);
	printf ("base :%s: + toto -> :%s:\n",base,fp);
}
int main (int, char *argv[])
{
	SAVEPATH save;
	tstmak ("");
	tstmak (".");
	tstmak ("..");
	tstmak ("../h");
	tstmak ("../../h");
	path_pushdir ("/home/solucor/jack",&save);
	show ();
	path_popdir  (&save);
	show ();
	path_chdir	("/usr/include");
	show ();
	path_chdir	("/home/solucor/jack/prj/tool");
	show ();
	path_chdir ("/");
	show ();
}
#endif
