#include <string.h>
#include <stdlib.h>
#include "vfs.h"


static char **tbstr;
static int    dirtype;
static int	  maxstr;
static int	  nbstr;

static int fct(
	const FILEINFO *info,
	const char *path,
	const char *add,
	int DUM1)
{
	if (info->type == dirtype && nbstr < maxstr){
		char fpath[100];
		path_make (path,add,fpath);
		path_make (fpath,info->name,fpath);
		path_simplifie (fpath,fpath);
		tbstr[nbstr] = strdup (fpath);
		nbstr++;
	}
	return 1;
}
/*
	Obtient la liste triee des fichiers repondant au critere select
	Retourne nombre de fichier trouve, chaque fichier est accumule
	dans tbstr. l'espace pour chaque entree de tbstr est alloue dynamiquement
	L'appelant est responsable de liberee l'espace (free).

	Conserve le path dans le mme format que celui propos par fpath
	(le path n'est pas convertie en absolue).
*/
int dir_getlist (
	const char *path,
	const char *select,		/* wildcard */
	int   nbsub,		/* nombre de niveau de sous-directory */
	char *tb_str[],
	int   max_str)		/* limite de tbstr */
{
	tbstr  = tb_str;
	maxstr = max_str;
	dirtype = FILETYPE_REG;
	nbstr  = 0;
	dir (path,select,nbsub,fct);
	tbstr_sort (tbstr,nbstr);
	return nbstr;
}
/*
	Obtient la liste triee des fichiers definit par fpath
	Retourne nombre de fichier trouve, chaque fichier est accumule
	dans tbstr. l'espace pour chaque entree de tbstr est alloue dynamiquement
*/
int dir_getlistp (
	const char *fpath,
	int nbsub,
	char *tb_str[],
	int maxstr)
{
	char path[PATH_MAX];
	char file[PATH_MAX];
	path_splitlex (fpath,path,file);
	return dir_getlist (path,file,nbsub,tb_str,maxstr);
}

/*
	Obtient la liste triee des repertoire repondant au critere select
	Retourne nombre de repertoire trouve, chaque path de repertoire
	est accumule dans tbstr. l'espace pour chaque entree de tbstr est
	alloue dynamiquement. L'appelant est responsable de liberee l'espace
	(free).
*/
int dir_getlistd (
	const char *path,
	const char *select,		/* wildcard */
	int   nbsub,		/* nombre de niveau de sous-directory */
	char *tb_str[],
	int   max_str)		/* limite de tbstr */
{
	tbstr  = tb_str;
	maxstr = max_str;
	dirtype = FILETYPE_DIR;
	nbstr  = 0;
	dir (path,select,nbsub,fct);
	tbstr_sort (tbstr,nbstr);
	return nbstr;
}
/*
	Obtient la liste triee des repertoires definit par fpath
	Retourne nombre de repertoire trouve, chaque repertoire est accumule
	dans tbstr. l'espace pour chaque entree de tbstr est alloue dynamiquement
*/
int dir_getlistdp (
	const char *fpath,
	int nbsub,
	char *tb_str[],
	int maxstr)
{
	char path[100];
	char file[20];
	path_splitlex (fpath,path,file);
	return dir_getlistd (path,file,nbsub,tb_str,maxstr);
}

#ifdef TEST

int main (int argc, char *argv[])
{
	char *tbstr[200];
	int nbstr = dir_getlist ("","*.c",0,tbstr,200);
	int i;
	printf ("nbstr = %d\n",nbstr);
	for (i=0; i<nbstr; i++){
		printf ("%s\n",tbstr[i]);
		free (tbstr[i]);
	}
	printf ("test avec chaine vide\n");
	nbstr = dir_getlistp ("",0,tbstr,200);
	printf ("nbstr = %d\n",nbstr);
	for (i=0; i<nbstr; i++){
		printf ("%s\n",tbstr[i]);
		free (tbstr[i]);
	}
	printf ("test avec path relatif\n");
	nbstr = dir_getlistp ("../sunos/*.c",0,tbstr,200);
	printf ("nbstr = %d\n",nbstr);
	for (i=0; i<nbstr; i++){
		printf ("%s\n",tbstr[i]);
		free (tbstr[i]);
	}
	printf ("test avec path invalide\n");
	nbstr = dir_getlistp ("e:/kit/groupe/xxx/*.ddl",0,tbstr,200);
	printf ("nbstr = %d\n",nbstr);
	for (i=0; i<nbstr; i++){
		printf ("%s\n",tbstr[i]);
		free (tbstr[i]);
	}
	return 0;
}

#endif
